<?php
/*******************************************************************************
 * 
 * For information related to this plugin, see config.php.
 *
 *******************************************************************************/



/* 
 * Purpose:
 * Cacti architecture uses hooks to enable plugins to be added to 
 * Cacti.  This is the function which defines which hooks provided
 * by Cacti plugin architecture will be used, and which function
 * defines the user-defined (i.e. the new plugin) behavior/
 * functionality for each hook specified.
 *
 */
function plugin_init_telix() {
	global $plugin_hooks;


	$plugin_hooks['top_header_tabs']['telix'] = 'telix_show_tab';

	$plugin_hooks['top_graph_header_tabs']['telix'] = 'telix_show_tab';

	$plugin_hooks['config_arrays']['telix'] = 'telix_config_arrays';

	$plugin_hooks['draw_navigation_text']['telix'] = 'telix_draw_navigation_text';

	$plugin_hooks['config_settings']['telix'] = 'telix_config_settings';

}



/*
 * Purpose:
 * Defines version/general information for Telix Cacti Plugin.
 * The actual values are read from Telix/config.php.
 */
function telix_version () {
      global $telix_name, $telix_version, $telix_long_name,
             $telix_author, $telix_homepage, $telix_email,
             $telix_url, $config;
	     
	// Includes config.php file to read configuration settings
 	// in it.
      include($config["base_path"] . "/plugins/telix/config.php");	     

	return array('name' 		=> $telix_name,
			 'version' 		=> $telix_version,
			 'longname'		=> $telix_long_name,
			 'author'		=> $telix_author,
			 'homepage'		=> $telix_homepage,
			 'email'		=> $telix_email,
			 'url'		=> $telix_url
			 );
}



/* 
 * Purpose: 
 * Displays a Telix tab if it is configured to display and if
 * the user has the permission to view this tab.
 *
 * Reference:
 * - Also see config.php for $telix_tab value.
 */
function telix_show_tab () {
	global $config, $user_auth_realms, $user_auth_realm_filenames, 
             $telix_tab;

	     
	// Includes database.php to enable use of database access 
	// functions provided with Cacti.  
	// Includes Telix/config.php to read configurations settings in it.
	include_once($config["library_path"] . "/database.php");
	include_once($config["base_path"] . "/plugins/telix/config.php");
	

	// If the Telix tab is configured not to be displayed.
	if (!$telix_tab)
		return;


	// Check if the user can access main Telix page.  The user can 
      // access the main Telix page if the user has the permission to
      // access either (or both) the router interface management 
	// functionality or router configuration files management 
	// functionality.
	$realm_id2 = 0;
	if (isset($user_auth_realm_filenames{basename('telix_ifs.php')})) {
		$realm_id2 = $user_auth_realm_filenames{basename('telix_ifs.php')};
	}

	$realm_id3 = 0;
	if(isset($user_auth_realm_filenames{basename('telix_dialer_strs.php')})) {
		$realm_id3 = $user_auth_realm_filenames{basename('telix_dialer_strs.php')};	 	
  	}

	$realm_id4 = 0;
	if(isset($user_auth_realm_filenames{basename('telix_ifs_view.php')})) {
		$realm_id4 = $user_auth_realm_filenames{basename('telix_ifs_view.php')};	 	
  	}

	if ((db_fetch_assoc("select user_auth_realm.realm_id
	     from user_auth_realm where user_auth_realm.user_id='" . $_SESSION["sess_user_id"] . "'
	     and user_auth_realm.realm_id='$realm_id2'")) || 
          (db_fetch_assoc("select user_auth_realm.realm_id
	     from user_auth_realm where user_auth_realm.user_id='" . $_SESSION["sess_user_id"] . "'
           and user_auth_realm.realm_id='$realm_id3'")) ||
	    (db_fetch_assoc("select user_auth_realm.realm_id
	     from user_auth_realm where user_auth_realm.user_id='" . $_SESSION["sess_user_id"] . "'
           and user_auth_realm.realm_id='$realm_id4'")) ) {

		print '<a href="' . $config['url_path'] . 'plugins/telix/telix.php"><img src="' . $config['url_path'] . 
                  'plugins/telix/images/tab_telix.gif" alt="Telix" align="absmiddle" border="0"></a>';
	}
}



/* 
 * Purpose:
 * Defines/Updates array used by both Cacti and Telix plugin.
 * Specifically, it adds two realms to enable the control of 
 * user access to different funtionalities of the Telix Cacti 
 * plugin.  There are currently two functionalities, namely
 * Router Interfaces Management and Router Configuration Files
 * management.
 * 
 */
function telix_config_arrays () {
	global $user_auth_realms, $user_auth_realm_filenames, $menu, 
             $config, $telix_tab;


	// Includes config.php file to read configuration settings
 	// in it.
	include($config["base_path"] . "/plugins/telix/config.php");     
	     
	// Assign a realm id for Telix plugin.  This will enable
      // user permission management for Telix plugin.
	$user_auth_realms[$telix_ifs_realm_id]='Telix - Router Interfaces Mgt';
	$user_auth_realms[$telix_dialer_strs_realm_id]='Telix - Router Dialer Strings Mgt';
	$user_auth_realms[$telix_ifs_view_realm_id]='Telix - View Router Interfaces Status';
	$user_auth_realm_filenames['telix_ifs.php'] = $telix_ifs_realm_id;
	$user_auth_realm_filenames['telix_dialer_strs.php'] = $telix_dialer_strs_realm_id;
	$user_auth_realm_filenames['telix_ifs_view.php'] = $telix_ifs_view_realm_id;
	

	// If Telix tab is configured not to be displayed, display
      // the Telix menu item under Utitlies menu.  
	// Insert the Telix menu item in front of the Logout menu item.
	if (!$telix_tab) {
		$temp = $menu["Utilities"]['logout.php'];
		unset($menu["Utilities"]['logout.php']);
		$menu["Utilities"]['plugins/telix/telix.php'] = "Telix";
		$menu["Utilities"]['logout.php'] = $temp;
	}


}


/*
 * Purpose:
 * Defines navigation path/control for each Telix plugin page.
 * The navigation path is usually displayed near top of a page.
 * 
 */
function telix_draw_navigation_text ($nav) {

	global $config;


	// Includes config.php file to read configuration settings
 	// in it.
      include($config["base_path"] . "/plugins/telix/config.php");	

	// telix.php.
	$nav["telix.php:"] = array("title" => "Telix", 
                                 "mapping" => "index.php:", 
                                 "url" => "telix.php", 
                                 "level" => "1");

	
      // telix_ifs.php
	$nav["telix_ifs.php:"] = 
          array("title" => $telix_ifs_task, 
                "mapping" => "index.php:,telix.php:", 
                "url" => "telix_ifs.php", 
                "level" => "2");

      // telix_ifs.php?action=actions
      $nav["telix_ifs.php:actions"] =
          array("title" => "(Actions)", 
                "mapping" => "index.php:,telix.php:,telix_ifs.php:", 
                "url" => "telix_ifs.php", 
                "level" => "3");

	
      // telix_dialer_strs.php
	$nav["telix_dialer_strs.php:"] = 
          array("title" => $telix_dialer_strs_task , 
                "mapping" => "index.php:,telix.php:", 
                "url" => "telix_dialer_strs.php", 
                "level" => "2");


      // telix_ifs_view.php
	$nav["telix_ifs_view.php:"] = 
          array("title" => $telix_ifs_view_task, 
                "mapping" => "index.php:,telix.php:", 
                "url" => "telix_ifs_view.php", 
                "level" => "2");


      return $nav;

}



/*
 * Purpose: 
 * Defines settings related to Telix plugin.
 * These settings are dipslayed in Settings/Telix tab.  The settings
 * are used to change the behavior of the plugin.  For example,
 * to change the IP address of the router to communicate with.
 *
 */
function telix_config_settings () {
	global $tabs, $settings, $telix_tftp_enabled, $config;
     

	include($config["base_path"] . "/plugins/telix/config.php"); 


	// Add a new tab in settings.php page.
	$tabs["telix"] = "Telix";
	

	// Settings (in Settings section of Cacti) related to Router Interfaces
      // Management.
	$ifsMgtSettings = 
		array(
	
			"telix_ifs_header" => 
				array(
					"friendly_name" => "Router Interfaces Management Settings",
					"method" => "spacer",
				),

			"telix_ifs_snmp_version" => 
				array(
					"friendly_name" => "SNMP Version",
					"description" => "The SNMP version to be used.",
					"method" => "drop_array",
					"array" => array(0 => 'Version 1', 1 => 'Version 2c', 2 => 'Version 3'),
					"default" => 0
				),

			"telix_ifs_ro" => 
				array(
					"friendly_name" => "RO Community String",
					"description" => "RO Community string to be used for SNMP walk and get functions.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_ifs_rw" => 
				array(
					"friendly_name" => "RW Community String",
					"description" => "RW Community string to be used for SNMP set function.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_ifs_router_ip" => 
				array(
					"friendly_name" => "Router IP Address",
					"description" => "IP Address of the target router.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_ifs_oid" => 
				array(
					"friendly_name" => "Interfaces OID",
					"description" => "The OID used to perform SNMP walk function which extracts all information related to interfaces.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_ifs_admin_oid" => 
				array(
					"friendly_name" => "Admin Status OID",
					"description" => "The OID for Admin Status (without interface index which will be appended at the time of function call).",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_ifs_if_desc_oid" => 
				array(
					"friendly_name" => "Interfaces Description OID",
					"description" => "The OID for Interfaces Description.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_ifs_default_refresh_rate" => 
				array(
					"friendly_name" => "Page Refresh Interval",
					"description" => "The default time interval (in seconds) at which the Router Interfaces Management page is refreshed, e.g., every 30 seconds.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),
			"telix_ifs_rows_per_page" => 
				array(
					"friendly_name" => "Rows Displayed Per Page",
					"description" => "The number of rows displayed per page.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				)
		);


	// Settings (in Settings section of Cacti) related to View Router Interfaces
      // Status.
	$ifsViewSettings = 
		array(
	
			"telix_ifs_view_header" => 
				array(
					"friendly_name" => "View Router Interfaces Status Settings",
					"method" => "spacer",
				),

			"telix_ifs_view_snmp_version" => 
				array(
					"friendly_name" => "SNMP Version",
					"description" => "The SNMP version to be used.",
					"method" => "drop_array",
					"array" => array(0 => 'Version 1', 1 => 'Version 2c', 2 => 'Version 3'),
					"default" => 0
				),

			"telix_ifs_view_ro" => 
				array(
					"friendly_name" => "RO Community String",
					"description" => "RO Community string to be used for SNMP walk and get functions.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_ifs_view_router_ip" => 
				array(
					"friendly_name" => "Router IP Address",
					"description" => "IP Address of the target router.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_ifs_view_oid" => 
				array(
					"friendly_name" => "Interfaces OID",
					"description" => "The OID used to perform SNMP walk function which extracts all information related to interfaces.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_ifs_view_if_desc_oid" => 
				array(
					"friendly_name" => "Interfaces Description OID",
					"description" => "The OID for Interfaces Description.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_ifs_view_default_refresh_rate" => 
				array(
					"friendly_name" => "Page Refresh Interval",
					"description" => "The default time interval (in seconds) at which the View Router Interfaces Status page is refreshed, e.g., every 30 seconds.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),
			"telix_ifs_view_rows_per_page" => 
				array(
					"friendly_name" => "Rows Displayed Per Page",
					"description" => "The number of rows displayed per page.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				)
		);



	// Settings (in Settings section of Cacti) related to Router 
      // Dialer Strings Management.
	$dialerStrsMgtSettings = 
		array(
	
			"telix_dialer_strs_header" => 
				array(
					"friendly_name" => "Router Dialer Strings Management Settings",
					"method" => "spacer",
				),

			"telix_dialer_strs_snmp_version" => 
				array(
					"friendly_name" => "SNMP Version",
					"description" => "The SNMP version to be used.",
					"method" => "drop_array",
					"array" => array(0 => 'Version 1', 1 => 'Version 2c', 2 => 'Version 3'),
					"default" => 0
				),

			"telix_dialer_strs_ro" => 
				array(
					"friendly_name" => "RO Community String",
					"description" => "RO Community string to be used for SNMP walk and get functions.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_dialer_strs_router_ip" => 
				array(
					"friendly_name" => "Router IP Address",
					"description" => "IP Address of the target router.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_dialer_strs_oid" => 
				array(
					"friendly_name" => "Interfaces OID",
					"description" => "The OID used to perform SNMP walk function which extracts all information related to interfaces.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_dialer_strs_if_desc_oid" => 
				array(
					"friendly_name" => "Interfaces Description OID",
					"description" => "The OID for Interfaces Description.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),

			"telix_dialer_strs_default_refresh_rate" => 
				array(
					"friendly_name" => "Page Refresh Interval",
					"description" => "The default time interval (in seconds) at which the Router Dialer Strings Management page is refreshed, e.g., every 30 seconds.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),
			"telix_dialer_strs_rows_per_page" => 
				array(
					"friendly_name" => "Rows Displayed Per Page",
					"description" => "The number of rows displayed per page.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),
			"telix_dialer_strs_telnet_username" => 
				array(
					"friendly_name" => "Telnet Account Username",
					"description" => "Username of the user account used to telnet the target router.",
					"method" => "textbox",
					"size" => 50,
					"max_length" => 255,
				),
			"telix_dialer_strs_telnet_password" => 
				array(
					"friendly_name" => "Telnet Account Password",
					"description" => "Password of the user account used to telnet the target router." .
							     "(Please enter the password twice to confirm.)",
					"method" => "textbox_password",
					"size" => 50,
					"max_length" => 255,
				),
	
		);



	// Add settings related to Router Interfaces Management.
	$settings["telix"] = $ifsMgtSettings;


	// Add settings related to View Router Interfaces Status.
	$settings["telix"] = array_merge($settings["telix"], $ifsViewSettings );


	// Add settings related to Router Dialer Strings Management.
	$settings["telix"] = array_merge($settings["telix"], $dialerStrsMgtSettings );



}


?>