<?php
/*******************************************************************************

    Program: Telix.

*******************************************************************************/


// Constants

// ---
// Defines maximum number of pages to be displayed when showing all router 
// interfaces.
define("MAX_DISPLAY_PAGES", 21);


// ---
// Keys to index and extract interface status information.
define("INTERFACE_INDICES_KEY", "interface_indices");
define("INTERFACE_DESC_KEY", "interface_desc");
define("INTERFACE_TYPE_KEY", "interface_type");
define("INTERFACE_ADMIN_STATUS_KEY", "interface_admin_status");
define("INTERFACE_OP_STATUS_KEY", "interface_op_status");
define("INTERFACE_DIALER_IDS_KEY", "interface_dialer_ids");
define("INTERFACE_DIALER_STRS_KEY", "interface_dialer_strs");
define("INTERFACE_ERR_STR", "interface_err_string");



// ---
// Define constants related to SNMP function calls.
define("SNMPv1", 0);
define("SNMPv2c", 1);
define("SNMPv3", 2);
define("DELIMITER_OFFSET", 2);   // for ": " string in snmpwalk output.


// ---
// Error messages.
define("ERR_SETTINGS", "Incorrect SNMP Settings in Settings/Telix tab");
define("ERR", "Error during SNMP function call");
define("ERR_NO_SELECTION", "Error: Did not select an interface to update.");
define("ERR_MSG_STR", "err_msg");
define("ROUTER_ERR_DIAL_STR_NOT_SET", "<font color='red'> Not Configured </font>");


// ---
// Factory default of page refresh interval.
// Note that this value is used only when the user did not
// set the default page refresh rate (for Router Dialer Strings Management)
// through Configuration > Settings > Telix tab.page.
define("FACTORY_DEFAULT_DIALER_STRS_REFRESH_RATE", 300);


// ---
// Factory default of the number of rows displayed on this page.
define("FACTORY_DEFAULT_ROWS_PER_PAGE", 15);


// ---
// Used for the PHP library function stream_set_blocking.
define("STREAM_NON_BLOCKING", 0);


// ---
// Constants used by Router Dial Strings Management functionality.

// Interface information of Dialer interfaces has the following 
// format string.  Note that the format string is a regular
// expression.
define("DIALER_IF_INFO_STR_FORMAT", "Dialer([0-9]*)");

// Used to specify file handles which are created as part of
// proc_open function call.
define("PROCESS_PIPE_TYPE_PIPE", "pipe");
define("PROCESS_PIPE_MODE_READ", "r");
define("PROCESS_PIPE_MODE_WRITE", "w");

// Used to index the array created by proc_open which contains
// file handles to stdin, stdout and stderr.
define("PROCESS_PIPE_STDIN", 0);
define("PROCESS_PIPE_STDOUT", 1);
define("PROCESS_PIPE_STDERR", 2);

// Used to parse telnet output (sent to stdout).
define("TELNET_PROMPT_USERNAME", "Username:");
define("TELNET_PROMPT_PASSWORD", "Password:");
define("TELNET_PROMPT_CMD", "CR9TLX01#");
define("TELNET_PROMPT_CMD_CONFIG", "CR9TLX01(config)#");
define("TELNET_PROMPT_CMD_CONFIG_IF", "CR9TLX01(config-if)#");
define("TELNET_PROMPT_MORE", "--More--");
define("TELNET_PROMPT_AUTHENTICATION_FAILED", "% Authentication failed");
define("STR_COMPARE_EQUALS", 0);
define("REG_PATTERN_MATCH_INDEX", 1);

// Number of telnet login attempts.  Set to one less than the total
// number allowed (which is 3) so that Telix initiates the disconnect.
define("TELNET_LOGIN_ATTEMPTS", 2);  

// Constants related to telnet session in general
define("TELNET_SESSION_CLEANUP_DURATION", 1);   // in seconds
define("TELNET_SESSION_END_STR", "0");   // telnet output after sending exit command

// Used to construct and issue commands through telnet.
define("ROUTER_CMD_SHOW_IF", "show running-config interface dialer ");
define("ROUTER_CMD_CONFIG", "configure terminal");
define("ROUTER_CMD_IF", "interface dialer ");
define("ROUTER_CMD_SET_IF_STR", "dialer string ");
define("ROUTER_CMD_UNSET_IF_STR", "no dialer string ");
define("TELNET_CMD_QUIT", "exit");
define("TELNET_CMD_SPACE", " "); 

// Used to parse details of a particular interface returned after
// executing the "show running-config interface dialer <id>" command.
define("ROUTER_DIALER_STR_KEY", "dialer string");
define("ROUTER_START_CONFIG_MARKER", "!");
define("ROUTER_END_CONFIG_MARKER", "end");
define("ROUTER_DIALER_STR_FORMAT", "dialer string ([0-9]*)");

// Used to cache dialer strings.
define("SESS_DIALER_STRS_KEY", "dialer_strs");
define("SESS_DIALER_STRS_TIMESTAMP_KEY", "dialer_strs_timestamp");
define("SESS_DIALER_STRS_TIME_KEY", "dialer_strs_time");
define("CUR_TIMESTAMP_FORMAT", "d-m-Y   H:i:s T");
define("DIALER_STRS_REFRESH_INTERVAL", 1800);   // 30 minutes
//define("DIALER_STRS_REFRESH_INTERVAL", 60);   // for testing only



// ---
// Numeric representation of each available action that can be
// performed on an interface.  See variable $interface_actions. 
define("ACTION_SET_DIALER_STRING", 1);




// Define actions that can be performed on interface.
// Currently supported actions:
//    - "Set Dialer String"
//       - Changes the dialer string of a dialer interface.
$interface_actions = array(
	ACTION_SET_DIALER_STRING => "Set Dialer String"
	);



// Change current working directory to Cacti root directory.  This is
// required to allow existing Cacti PHP files like auth.php etc. to be used
// in plugin code, as include statements in existing Cacti PHP files assumes
// that current working directory is the Cacti root directory (i.e. relative
// paths (relative to Cacti root directory) are used in include statements).  
// This also enables reuse of existing Cacti code with minimal changes.
chdir('../../');


// Authenticate the user.  Also includes ./include/config.php.
include_once("./include/auth.php");



// Process the action selected by the user.  If no action is 
// selected, just display the page.
// Set default action 
if (!isset($_REQUEST["action"])) { $_REQUEST["action"] = ""; }

switch ($_REQUEST["action"]) {

	case 'actions':
		form_actions();

		break;

	default:
		$refresh = get_refresh_interval();
		set_session_variables();
		$interface_status = get_interface_status();	
		$interface_status = trim_none_dialers($interface_status);	
		$interface_status_trimed = trim_array($interface_status);	
		include_once("./include/top_header.php");
		display_telix_dialer_strs_page($interface_status, $interface_status_trimed);
		include_once("./include/bottom_footer.php");
		
		break;
}


/*
 * Purpose: 
 *    Get refresh interval for this page.
 *
 * Inputs:
 *    None.
 * 
 * Returns:
 *    An array of two elements:
 *       {"page" => <name of this php page>, 
 * 	   {"seconds" => <refresh interval>}
 */
function get_refresh_interval() {

	global $settings, $config;

	include("plugins/telix/config.php");

	// Check if the user has changed the refresh rate for the session.
	// The new value is set through the View Router Interfaces Status page and 
	// not the Settings/Telix page.
	if( isset($_REQUEST["new_refresh_rate"]) ) {
		
		// Check to make sure that the user entered a numeric value.
		// If a non-numeric value is entered, it is ignored.
		if((is_numeric($_REQUEST["new_refresh_rate"])) &&
               ($_REQUEST["new_refresh_rate"] >= 1) ) {
			$_SESSION["sess_dialer_strs_page_refresh_rate"] = intval($_REQUEST["new_refresh_rate"]);
		} 
	}	


	// If refresh rate already set and set as session variable, use it.
	if( (isset($_SESSION["sess_dialer_strs_page_refresh_rate"])) && 
          ($_SESSION["sess_dialer_strs_page_refresh_rate"] >= 1) ) {
		$refresh["seconds"] = intval($_SESSION["sess_dialer_strs_page_refresh_rate"]);
		$refresh["page"] = "telix_dialer_strs.php";
		return $refresh;
	}


	include_once($config["library_path"] . "/database.php");

	// The condition for name column must match the field name 
	// specified in "telix_config_settings" function of 
	// plugings/telix/setup.php page.  That is, the field name 
	// assigned to Router Dialer Strings Management page refresh 
	// rate.
	$sqlQuery = "select value from settings where name='telix_dialer_strs_default_refresh_rate'";
	$default_refresh_rate = db_fetch_row($sqlQuery);

	// If user did not set default refresh rate or if refresh rate is set at
  	// less than or equals to 0, then factory default used.  Also remember
	// this value as session variable.
	if( (!isset($default_refresh_rate)) ||
          (!isset($default_refresh_rate["value"])) ||
	    (!is_numeric($default_refresh_rate["value"])) ||
          ($default_refresh_rate["value"] < 1) ) {
		
		// Remember the refresh rate. 
		$_SESSION["sess_dialer_strs_page_refresh_rate"] = FACTORY_DEFAULT_DIALER_STRS_REFRESH_RATE; 
		
		$refresh["seconds"] = $_SESSION["sess_dialer_strs_page_refresh_rate"];
		$refresh["page"] = "telix_dialer_strs.php";
		return $refresh;
	}
	
	
	// Display user-defined default.
	$_SESSION["sess_dialer_strs_page_refresh_rate"] = intval($default_refresh_rate["value"]);
	$refresh["seconds"] = $_SESSION["sess_dialer_strs_page_refresh_rate"];
	$refresh["page"] = "telix_dialer_strs.php";
	return $refresh;

}


/*
 * Purpose: 
 *    Setup session variables and destroy session variables used by 
 *    other php pages which conflict with session variables used by
 *    this php page.
 */
function set_session_variables() {

	// Setup session variables.
	// Remember which page is being displayed.	
	if(isset($_SESSION["sess_router_ifs_current_page"])) {
		unset($_SESSION["sess_router_ifs_current_page"]);
	}
	if(isset($_SESSION["sess_router_ifs_view_current_page"])) {
		unset($_SESSION["sess_router_ifs_view_current_page"]);
	}
	if(isset($_REQUEST["page_ifs"])) {
		unset($_REQUEST["page_ifs"]);
	}	
	if(isset($_REQUEST["page_ifs_view"])) {
		unset($_REQUEST["page_ifs_view"]);
	}

	load_current_session_value(
		"page_dialer_strs", "sess_router_dialer_strs_current_page", "1");

	load_current_session_value(
		"page_refresh_rate", "sess_dialer_strs_page_refresh_rate", "0");

	// This PHP page also uses the following three session variables 
	// which are maintained in other places in this page.
	//define("SESS_DIALER_STRS_KEY", "dialer_strs");
	//define("SESS_DIALER_STRS_TIMESTAMP_KEY", "dialer_strs_timestamp");
	//define("SESS_DIALER_STRS_TIME_KEY", "dialer_strs_time");

}



/*
 * Purpose: 
 *    Displays the Telix Router Dialer Strings Management page.
 */
function display_telix_dialer_strs_page($interface_status, $interface_status_trimed, $res="") {

	global $config, $colors, $interface_actions;

	include_once($config["library_path"] . "/database.php");


	// Display error message if it is set.
	if(!empty($res)) {
		$err_msg = "<tr><td> " .
			     "<font color='red'> <b> $res </b> </font><br>" .
			     "</td></tr>";

		html_start_box("", "98%", "FFFFFF", "3", "center", "");
		print $err_msg;
		html_end_box();
	}

	if(isset($interface_status[ERR_MSG_STR])) {
		$err_msg =  "<tr><td> " .
				"<font color='red'> <b> " .
	      	      $interface_status[ERR_MSG_STR] . "</b> </font><br>" .
				"</td></tr>";

		html_start_box("", "98%", "FFFFFF", "3", "center", "");
		print $err_msg;
		html_end_box();
	}

	echo "<br><br>";


	// Display task selection form.
	html_start_box("<strong>Tasks &nbsp&nbsp&nbsp&nbsp[task: Router Dialer Strings Management]</strong>", "98%", $colors["header"], "3", "center", "");

	include($config["base_path"] . "/plugins/telix/telix_select_task_form_dialer_strs.php");  

	html_end_box();


	// Display the section that allows user to choose the page refresh rate 
  	// and to invoke immediate manual refresh.
	html_start_box("<strong>Router Interfaces</strong>", "98%", $colors["header"], "3", "center", "");

	include("./plugins/telix/telix_refresh_form_dialer_strs.php");  

	html_end_box();


	// Useful notes.
	// Update "Last Refreshed" timestamp for dialer strings.
	$notes_msg = 
		"<tr><td> " .
		"<b>Notes:</b> " .
		"In order to successfully add a new dialer string for a " .
		"new interface, all dependent entries/settings " .
		"(e.g. dialer pool) must be set first." .
		"</td></tr>";


	html_start_box("", "98%", "FFFFFF", "3", "center", "");
	print($notes_msg);
	html_end_box();


	// Update "Last Refreshed" timestamp for dialer strings.
	
	$timestamp_msg = 
		"<tr><td> " .
		"<b>'Current Dialer String' column last refreshed at: </b>" .
		"&nbsp;&nbsp;&nbsp;" .
		"<font color='green'> ";

	if(isset($_SESSION[SESS_DIALER_STRS_TIMESTAMP_KEY])) {			
		$timestamp_msg .= $_SESSION[SESS_DIALER_STRS_TIMESTAMP_KEY];
	} else {
		$timestamp_msg .= "N/A";
	}
		
	$timestamp_msg .= " </font> </td></tr>";

	html_start_box("", "98%", "FFFFFF", "3", "center", "");
	print($timestamp_msg);
	html_end_box();


	// Display the section that shows the details of the router interfaces.
	html_start_box("", "98%", $colors["header"], "3", "center", "");


	// Count the number of rows to display.  
	//Rows with description "" are not displayed.
	$num_interfaces = count($interface_status[INTERFACE_INDICES_KEY]);
	
	// If number of interfaces found is greater than 0, then remove those
  	// (from the count) whose description is "" (since they are not to be
  	// displayed.
	if($num_interfaces > 0) {
		$ifs_desc_a = $interface_status[INTERFACE_DESC_KEY];

		foreach($ifs_desc_a as $key => $value) {

			if(empty($value)) {
				$num_interfaces--;
			}
		}
	}


	// generate page list
	$sqlQuery = "select value from settings where name='telix_dialer_strs_rows_per_page'";
	$res = db_fetch_row($sqlQuery);
	$telix_dialer_strs_num_rows = $res["value"];

	// If number of rows per page is not set, factory default is used.
	if( (!(isset($telix_dialer_strs_num_rows))) || (empty($telix_dialer_strs_num_rows)) ||
	    (!is_numeric($telix_dialer_strs_num_rows)) ||
          (intval($telix_dialer_strs_num_rows) < 1) ) {
		$telix_dialer_strs_num_rows = FACTORY_DEFAULT_ROWS_PER_PAGE;
	}

	$telix_dialer_strs_num_rows = intval($telix_dialer_strs_num_rows);

	$url_page_select = get_page_list($_REQUEST["page_dialer_strs"], MAX_DISPLAY_PAGES, $telix_dialer_strs_num_rows, $num_interfaces, "");

	
	// Generate and display the navigation part of the output.
	// The colspan here must match number of columns as defined in $display_text
	$nav = "<tr bgcolor='#" . $colors["header"] . "'>			
			<td colspan='8'>
				<table width='100%' cellspacing='0' cellpadding='0' border='0'>
					<tr>
						<td align='left' class='textHeaderDark'>
							<strong>&lt;&lt; "; if ($_REQUEST["page_dialer_strs"] > 1) { $nav .= "<a class='linkOverDark' href='telix_dialer_strs.php?page_dialer_strs=" . ($_REQUEST["page_dialer_strs"]-1) . "'>"; } $nav .= "Previous"; if ($_REQUEST["page_dialer_strs"] > 1) { $nav .= "</a>"; } $nav .= "</strong>
						</td>\n
						<td align='center' class='textHeaderDark'>
							Showing Rows " . (($num_interfaces>0)?(($telix_dialer_strs_num_rows*($_REQUEST["page_dialer_strs"]-1))+1): 0) . " to " . ((($num_interfaces < $telix_dialer_strs_num_rows) || ($num_interfaces < ($telix_dialer_strs_num_rows*$_REQUEST["page_dialer_strs"]))) ? $num_interfaces : ($telix_dialer_strs_num_rows*$_REQUEST["page_dialer_strs"])) . " of $num_interfaces [$url_page_select]
												
						</td>\n
						<td align='right' class='textHeaderDark'>
							<strong>"; if (($_REQUEST["page_dialer_strs"] * $telix_dialer_strs_num_rows) < $num_interfaces) { $nav .= "<a class='linkOverDark' href='telix_dialer_strs.php?page_dialer_strs=" . ($_REQUEST["page_dialer_strs"]+1) . "'>"; } $nav .= "Next"; if (($_REQUEST["page_dialer_strs"] * $telix_dialer_strs_num_rows) < $num_interfaces) { $nav .= "</a>"; } $nav .= " &gt;&gt;</strong>
						</td>\n
						<td align='right' class='textHeaderDark'>
						</td>\n	
					</tr>
				</table>
			</td>
		</tr>\n";

	print $nav;


	// Currently does not support sort by column.
	$display_text = array(
		"Index",
		"Description",
		"Type",
		"Admin Status",
            "Operational Status",
		"Current Dialer String",
		"New Dialer String",
		"");


	html_header($display_text);
	echo "<form action='telix_dialer_strs.php' method='post'>";


	// Get all status arrays.
	if($num_interfaces > 0) {
		$ifs_index_a = $interface_status_trimed[INTERFACE_INDICES_KEY];
		$ifs_desc_a = $interface_status_trimed[INTERFACE_DESC_KEY];
		$ifs_type_a = $interface_status_trimed[INTERFACE_TYPE_KEY];
		$ifs_admin_status_a = $interface_status_trimed[INTERFACE_ADMIN_STATUS_KEY];
		$ifs_op_status_a = $interface_status_trimed[INTERFACE_OP_STATUS_KEY];
		$ifs_dialer_ids_a = $interface_status_trimed[INTERFACE_DIALER_IDS_KEY];
		$if_dialer_strs_arr = $interface_status_trimed[INTERFACE_DIALER_STRS_KEY];

	} else {
		$ifs_index_a = array();
	}

	



	$i = 0;
	if (count($ifs_index_a) > 0) {
		foreach ($ifs_index_a as $key => $ifs_index) {

			if(!empty($ifs_desc_a[$ifs_index])) {
				form_alternate_row_color($colors["alternate"],$colors["light"],$i); $i++;
					?>
					<td>
						<?php print "$ifs_index"; ?>	
					</td>
					<td>
						<?php print "$ifs_desc_a[$ifs_index]"; ?>	
					</td>
					<td>						
						<?php print "$ifs_type_a[$ifs_index]"; ?>	
					</td>
					<td>				
						<?php print status_to_str($ifs_admin_status_a[$ifs_index]); ?>							
					</td>
					<td>
						<?php print status_to_str($ifs_op_status_a[$ifs_index]); ?>	
					</td>
					<td>
						<?php 
							if(isset($if_dialer_strs_arr[$ifs_dialer_ids_a[$ifs_index]])) {
								print $if_dialer_strs_arr[$ifs_dialer_ids_a[$ifs_index]]; 
							} else {
								$msg = "<font color='red'> " .
									 "Requires Refresh " .
									 "</font> ";
								print $msg;
							}
						?>
					</td>
					<td>
						<input type="text" size="25" 
							 name="<?php print "new_dialer_string_" . 
                                                             $ifs_dialer_ids_a[$ifs_index]; ?>"
							 align="right">
					</td>
					<td style="<?php print get_checkbox_style();?>" width="1%" align="right">
						<input type='radio' style='margin: 0px;' name='chk_ifs_index_selected' 
                                           value='<?php print $ifs_dialer_ids_a[$ifs_index]; ?>'>

						<?php

							// Generate hidden value which keeps track of current dialer
							// string if there is one.
							if(isset($if_dialer_strs_arr[$ifs_dialer_ids_a[$ifs_index]])) {
								if(strcmp(ROUTER_ERR_DIAL_STR_NOT_SET, 
									    $if_dialer_strs_arr[$ifs_dialer_ids_a[$ifs_index]]) !=
							   	STR_COMPARE_EQUALS) {
									$cur_dialer_str_val = 
										"<input type='hidden' " . 
								 		       "name='cur_dialer_str_" .	
											 $ifs_dialer_ids_a[$ifs_index] . "' " . 
                                        	 		 	      "value='" . 
											 $if_dialer_strs_arr[$ifs_dialer_ids_a[$ifs_index]] . 	
											 "'>";
									print $cur_dialer_str_val;

								} 
							}

						?>
	
					</td>
				</tr>
			<?php
			}
		}

		/* put the nav bar on the bottom as well */
		print $nav;
	}else{		

		$err_str = 
			"<tr><td colspan=7><em>No Router Interfaces Found." .
			"</em></td></tr>";

		
		print $err_str;

		$no_rows_displayed = true;


		// Clear session variables related to displaying page so that when data becomes
		// available, the first page is displayed.
		if(isset($_SESSION["sess_router_dialer_strs_current_page"])) {
			unset($_SESSION["sess_router_dialer_strs_current_page"]);
		}
		if(isset($_REQUEST["page_dialer_strs"])) {
			unset($_REQUEST["page_dialer_strs"]);
		}	

	}

	html_end_box(false);


	// Display the dropdown list containing a list of available 
	// actions for this form only if at least one row of data is 
	// displayed.
	if(!isset($no_rows_displayed)) {
		draw_actions_dropdown($interface_actions);
	} else {
		unset($no_rows_displayed);
	}

	print "</form>\n";
}




/*
 * Purpose: Obtain interface status using SNMP.
 * 
 * Returns:
 * Status of all interfaces on a router stored in the following format:
 * $interface_status = 
 *    array( 
 *       INTERFACE_INDICES_KEY => $interface_indices_a,
 *       INTERFACE_DESC_KEY => $interface_desc_a,
 *       INTERFACE_TYPE_KEY => $interafce_type_a,
 *       INTERFACE_ADMIN_STATUS_KEY => $interface_admin_status_a,
 *       INTERFACE_OP_STATUS_KEY => $interface_op_status_a);
 *     
 *
 * Notes:
 * - <TODO> 
 *    - Only SNMP v1 is currently covered.
 *
 * - This function is coded to a specific OID tree and thus can be
 *   considered as hardcoded.  The reason is that the PHP SNMP function
 *   snmpwalk returns an array of the following type:
 *   array = ("<type1>: <value1>", "<type2>: <value2>", ..., "<typeN>: <valueN>")
 *
 *   where <type1> = STRING, INTEGER etc.
 *   That is, it doesn't indicate which MIB object <value1> belongs to.
 *   Net-SNMP was used to determine what each value is.  For example, first MIB
 *   obejct is "ifNumber" which indicates the number of interface and second "set" of
 *   MIB objects is "ifIndex" of each interface available.
 *   As the result, the function is effectively hardcoded based on assumed data structure
 *   of the return value.
 *
 *   Also note that snmpwalk is used to take a snapshot of the state of all interfaces, 
 *   rather than using snmpget to extract state of each interface one at a time.  This
 *   is attempt to preserve the integrity of data.   
 */
function get_interface_status() {


	global $config;	

	$interface_status = array();


	include_once($config["library_path"] . "/database.php");

	// Get SNMP Version, RO Community String and Router IP Address.
	$sqlQuery = "select value from settings where name='telix_dialer_strs_snmp_version'";
	$res = db_fetch_row($sqlQuery);
	$snmpVersion = $res["value"];

	$sqlQuery = "select value from settings where name='telix_dialer_strs_ro'";
	$res = db_fetch_row($sqlQuery);
	$roString = $res["value"];

	$sqlQuery = "select value from settings where name='telix_dialer_strs_router_ip'";
	$res = db_fetch_row($sqlQuery);
	$routerIP = $res["value"];

	$sqlQuery = "select value from settings where name='telix_dialer_strs_oid'";
	$res = db_fetch_row($sqlQuery);
	$oid = $res["value"];

	$sqlQuery = "select value from settings where name='telix_dialer_strs_if_desc_oid'";
	$res = db_fetch_row($sqlQuery);
	$if_desc_oid = $res["value"];


	if( !(isset($snmpVersion)) ) {
		$res_array = array(INTERFACE_INDICES_KEY=>array());
		$res_array[ERR_MSG_STR] = 
			"Error: 'SNMP Version' setting for Router Dialer " .
			"Strings Management in " .
			"Configuration > Settings > Telix tab is not " .
			"set.";

		return $res_array;
	}

	if( (!(isset($roString))) || (empty($roString)) ) {
		$res_array = array(INTERFACE_INDICES_KEY=>array());
		$res_array[ERR_MSG_STR] = 
			"Error: 'RO Community String' setting for Router " .
			"Dialer Strings Management in " .
			"Configuration > Settings > Telix tab " .
			"is not set.";

		return $res_array;
	}

	if( (!(isset($routerIP))) || (empty($routerIP)) ) {
		$res_array = array(INTERFACE_INDICES_KEY=>array());
		$res_array[ERR_MSG_STR] = 
			"Error: 'Router IP Address' setting for Router " .
			"Dialer Strings Management in " .
			"Configuration > Settings > Telix tab " .
			"is not set.";

		return $res_array;
	}

	if( (!(isset($oid))) || (empty($oid)) ) {
		$res_array = array(INTERFACE_INDICES_KEY=>array());
		$res_array[ERR_MSG_STR] = 
			"Error: 'Interface OID' setting for Router Dialer " .
			"Strings Management in " .
			"Configuration > Settings > Telix tab is " .
			"not set.";

		return $res_array;
	}

	if( (!(isset($if_desc_oid))) || (empty($if_desc_oid)) ) {
		$res_array = array(INTERFACE_INDICES_KEY=>array());
		$res_array[ERR_MSG_STR] = 
			"Error: 'Interfaces Description OID' setting for " .		
			"Router Dialer Strings Management in " .
			"Configuration > Settings > Telix tab is not set.";

		return $res_array;
	}


	if($snmpVersion == SNMPv1) {
		// Also note that snmpwalk function takes two more optional parameters
  		// at the end, namely the timeout and retries which are not used at the
		// moment.
		// The '@' sign is to suppress any error or warnings being displayed to 
		// the page.  The program should handle it instead.
		$interfaceData = @snmpwalk($routerIP, $roString, $oid);

		if($interfaceData == FALSE) {
			$res_array = array(INTERFACE_INDICES_KEY=>array());
			$res_array[ERR_MSG_STR] = 
				"Error: 'snmpwalk' function call to extract " .		
				"interfaces information failed.  Please try " .
				"again.  If this function call continues to " .
				"fail, please check " .
				"Router Dialer Strings Management settings in " .
				"Configuration > Settings > Telix tab.";

			return $res_array;
		}

		
		// Get interfaces description.
		$interfaceDesc = @snmpwalk($routerIP, $roString, $if_desc_oid);		

		if($interfaceDesc == FALSE) {
			$res_array = array(INTERFACE_INDICES_KEY=>array());
			$res_array[ERR_MSG_STR] = 
				"Error: 'snmpwalk' function call to extract " .
				"interfaces description failed.  Please try " .
				"again.  If this function call continues to " .
				"fail, please check " .
				"Router Dialer Strings Management settings in " .
				"Configuration > Settings > Telix tab.";

			return $res_array;
		}


	} elseif ($snmpVersion == SNMPv2c) {
		// TODO
	
	} elseif ($snmpVersion == SNMPv3) {
		// TODO

	}


	// Get the number of interfaces.
	$str = $interfaceData[0];
	$numInterfaces = (int) substr(strrchr($str, ":"), DELIMITER_OFFSET);

	// Setup offset values.  This is based on the OID tree.
	$ifIndexOffset = 1;
	$ifTypeOffset = 2;
	$ifAdminStatusOffset = 7;
	$ifOperStatusPffset = 8;


	// Get interface index.
	$startPos = $ifIndexOffset;
	$interface_indices_a = array();
	for($i=$startPos; $i <= $numInterfaces; $i++) {
		$str = $interfaceData[$startPos+($i-1)];		
		$interface_indices_a[$i] = substr(strrchr($str, ":"), DELIMITER_OFFSET);
	}
	$interface_status[INTERFACE_INDICES_KEY] = $interface_indices_a;


	// Get interface type.
	$startPos = (($ifTypeOffset-1)* $numInterfaces) + 1;
	$interface_type_a = array();
	for($i=1; $i <= $numInterfaces; $i++) {
		$str = $interfaceData[$startPos+($i-1)];		
		$str = substr(strrchr($str, ":"), DELIMITER_OFFSET);
		// Notes: 
		// At the moment type is a number in string format (stored in $str).
		// If they need to be converted to text format, define the
		// conversion in the following function. 
		$str = type_to_str($str);
		$interface_type_a{$interface_indices_a[$i]} = $str;
	}
	$interface_status[INTERFACE_TYPE_KEY] = $interface_type_a;



	// Get interface admin status.
	$startPos = (($ifAdminStatusOffset-1) * $numInterfaces) + 1;
	$interface_admin_status_a = array();
	for($i=1; $i <= $numInterfaces; $i++) {
		$str = $interfaceData[$startPos+($i-1)]; // str = "up(1)" or "up(2)".		
		$str = substr(strrchr($str, "("), 1, 1); 
		$interface_admin_status_a{$interface_indices_a[$i]} = $str;
	}
	$interface_status[INTERFACE_ADMIN_STATUS_KEY] = $interface_admin_status_a;
	

	
	// Get interface operation status.
	$startPos = (($ifOperStatusPffset-1) * $numInterfaces) + 1;
	$interface_op_status_a = array();
	for($i=1; $i <= $numInterfaces; $i++) {
		$str = $interfaceData[$startPos+($i-1)]; // str = "up(1)" or "up(2)".		
		$str = substr(strrchr($str, "("), 1, 1); 
		$interface_op_status_a{$interface_indices_a[$i]} = $str;
	}
	$interface_status[INTERFACE_OP_STATUS_KEY] = $interface_op_status_a;


	// Get interface description.
	$interface_desc_a = array();
	for($i=0; $i < $numInterfaces; $i++) {
		$str = $interfaceDesc [$i];		
		$interface_desc_a{$interface_indices_a[$i+1]} = substr(strrchr($str, ":"), DELIMITER_OFFSET);
	}
	$interface_status[INTERFACE_DESC_KEY] = $interface_desc_a;


	return $interface_status;

}


/* 
 * Purpose: 
 * This is where value of "ifType" can be converted
 * from numeric format to string format (e.g. Serial1/0).
 * At the moment no conversion is performed.
 */
function type_to_str($str) {
	return $str;
}


function status_to_str($status) {
	
	$status_str = "Unknown";

	if($status == 1) {
		$status_str = '<font color="green">Up</font>';
	} elseif($status == 2) {
		$status_str = '<font color="red">Down</font>';
	} elseif($status == 3) {
		$status_str = '<font color="#ED7600">Testing</font>';
	} elseif($status == 4) {
		$status_str = '<font color="red">Unknown</font>';
	} elseif($status == 5) {
		$status_str = '<font color="#ED7600">Dormant</font>';
	} elseif($status == 6) {
		$status_str = '<font color="red">Not Present</font>';
	} elseif($status == 7) {
		$status_str = '<font color="red">Lower Layer Down</font>';
	}

	return $status_str;

}



/*
 * Purpose: 
 *    Process action selected by the user.
 *
 */
function form_actions() {

 	if($_POST["drp_action"] == strval(ACTION_SET_DIALER_STRING)) {
		// drop action == Set Dialer String
		$res = form_action_set_dialer_string();
		
	}


	// Already processed the request.  Unset action.
	unset($_REQUEST["action"]);	


	// Display the page.
	$refresh = get_refresh_interval();
	set_session_variables();
	$interface_status = get_interface_status();	
	$interface_status = trim_none_dialers($interface_status);	
	$interface_status_trimed = trim_array($interface_status);	
	include_once("./include/top_header.php");

	if(isset($res)) {
		display_telix_dialer_strs_page($interface_status, $interface_status_trimed, $res);
	} else {
		display_telix_dialer_strs_page($interface_status, $interface_status_trimed);
	}

	include_once("./include/bottom_footer.php");

}


/*
 * Purpose:
 *    Process user action related to "Set Dialer String.
 *
 * Returns:
 *    Error message string.  The error message string is an empty string if
 *    processing is successful.
 */
function form_action_set_dialer_string() {

	// Get the id of the dialer interface selected.
	$selectedInterface = @$_REQUEST["chk_ifs_index_selected"];

	if( !isset($selectedInterface) ) {
		return ERR_NO_SELECTION;
	}

	// Get the current/old dialer string of the dialer interface
	// selected.
	if(isset($_REQUEST["cur_dialer_str_$selectedInterface"])) {
		$cur_dialer_str = $_REQUEST["cur_dialer_str_$selectedInterface"];
	} else {
		$cur_dialer_str = "";
	}

	// Get the new dialer string that the user wish to set for the
	// selected interface.
	if(isset($_REQUEST["new_dialer_string_$selectedInterface"])) {
		$new_dialer_str = 
			$_REQUEST["new_dialer_string_$selectedInterface"];
		
		// Removes leading and trailing whitespaces.
		$new_dialer_str = trim($new_dialer_str);   

		// The new dialer string is empty so return.  Do not
		// allow the user to set dialer string to empty.
		if(empty($new_dialer_str)) {
			$err_msg = 
				"Error: New dialer string is either an empty " .
				"string or a string that only contains " .
			  	"whitespace characters.";

			return $err_msg;
		}


		// Error Checking: The dialer string needs to be numeric.
		if(!is_numeric($new_dialer_str)) {
			$err_msg = 
				"Error: Dialer string is not a numeric string.";

			return $err_msg;

		}

	}


	// Get required Telix settings.
	// Get router IP address.
	$sqlQuery = "select value from settings where name='telix_dialer_strs_router_ip'";
	$res = db_fetch_row($sqlQuery);
	$routerIP = $res["value"];

	// Get username of the Telnet account setup in the router that is
	// to be used by Telix.
	$sqlQuery = "select value from settings where name='telix_dialer_strs_telnet_username'";
	$res = db_fetch_row($sqlQuery);
	$username = $res["value"];

	// Get corresponding password of the Telnet account setup in the
	// router that is to be used by Telix.
	$sqlQuery = "select value from settings where name='telix_dialer_strs_telnet_password'";
	$res = db_fetch_row($sqlQuery);
	$password = $res["value"];

	// Telix settings error checking.
	if( (!(isset($routerIP))) || (empty($routerIP)) ) {
		return "Error: IP Address of the target router is not set in Settings/Telix tab.";
	}

	if( (!(isset($username))) || (empty($username)) ) {
		return "Error: Username of the Telnet account is not set in Settings/Telix tab.";
	}

	if( (!(isset($password))) || (empty($password)) ) {
		return "Error: Password of the Telnet account is not set in Settings/Telix tab.";
	}


	// Specify stdin, stdout and stderr to be used by the child 
	// process that will be created to run telnet.
	$descriptor_spec = 
		array(
			0 => array(PROCESS_PIPE_TYPE_PIPE, 
				     PROCESS_PIPE_MODE_READ),	// stdin
			1 => array(PROCESS_PIPE_TYPE_PIPE, 
			  	     PROCESS_PIPE_MODE_WRITE), // stdout		
			2 => array(PROCESS_PIPE_TYPE_PIPE, 
				     PROCESS_PIPE_MODE_WRITE)	// stderr		
		);


	// Execute the telnet command and keep track of the resource
	// returned which represents the process.
	$telnet_cmd = "telnet " . $routerIP;
	$process = proc_open($telnet_cmd, $descriptor_spec, $pipes);

	if(is_resource($process)) {
	
		// $pipes now looks like this:
		// 0 => writeable handle connected to child stdin.
		// 1 => readable handle connected to child stdout.
		// 2 => readable handle connected to child stderr.


		// This function currently supports regular files and 
		// socket streams only.  Therefore, it doesn't seem 
		// to make much different whether to set streams to 
		// blocking mode or non-blocking mode.  
  		// However, this function  must be called with either
		// blocking or non-blocking mode or the page doesn't get
		// loaded within expected time (not sure it will ever be
		// loaded though if the function is not called.
		// Ideally the streams (stdout and stderr, possibly stdin) 
		// should be set to blocking mode so that when the function
		// fgets is called, it will block until a non-empty line
		// of string is read.  Otherwise a loop is needed to 
		// continously calling fgets until a non-empty string is 
		// read, which is the current behavior of Telix.  There
		// doesn't seem to be a way around that.
		// Also note that PHP manual (see entry on Process Control
		// Functions) specifies that "Process Control functions 
		// should not be enabled within a webserver environment
		// and unexpected results may happen if any Process 
		// Control functions are used within a webserver
		// environment".  Therefore, Process Control functions 
		// can't be used to creat blocking behavior to make 
		// Telix more efficient.  Also note that it also makes
		// the program much more complex.	
		stream_set_blocking($pipes[PROCESS_PIPE_STDIN], 
				   	  STREAM_NON_BLOCKING);
		stream_set_blocking($pipes[PROCESS_PIPE_STDOUT], 
					  STREAM_NON_BLOCKING);
		stream_set_blocking($pipes[PROCESS_PIPE_STDERR], 
					  STREAM_NON_BLOCKING);


		// Login.  The number of attempts is 1 less than the
		// total number allowed so that the connection is
		// not automatically closed by the other side. 
		for($i = 0; $i <= TELNET_LOGIN_ATTEMPTS; $i++) {

			// Wait username prompt and then enter the username.
			wait_for_str($pipes, TELNET_PROMPT_USERNAME, 
					 PROCESS_PIPE_STDOUT);
			$telnet_username = $username . "\n";
			fwrite($pipes[PROCESS_PIPE_STDIN], $telnet_username);	
			fflush($pipes[PROCESS_PIPE_STDIN]);		
		

			// Wait for Password prompt and then enter the password.
			wait_for_str($pipes, TELNET_PROMPT_PASSWORD, 
					 PROCESS_PIPE_STDOUT);
			$telnet_password = $password . "\n";
			fwrite($pipes[PROCESS_PIPE_STDIN], $telnet_password);	
			fflush($pipes[PROCESS_PIPE_STDIN]);	


			// Wait for command prompt.
			$isLoggedIn = wait_for_str_err(
						$pipes, TELNET_PROMPT_CMD, 
				  		PROCESS_PIPE_STDOUT, 
						TELNET_PROMPT_AUTHENTICATION_FAILED);

			if($isLoggedIn) {
				break;
			}

		}


		if($isLoggedIn) {
			// Run "configure terminal" command.
			// Notes: The trailing newline is required!
			$cmd = ROUTER_CMD_CONFIG . "\n";
			fwrite($pipes[PROCESS_PIPE_STDIN], $cmd);
			fflush($pipes[PROCESS_PIPE_STDIN]);	
		

			// Wait for command prompt.
			wait_for_str($pipes, TELNET_PROMPT_CMD_CONFIG, 
					 PROCESS_PIPE_STDOUT); 


			// Run "interface dialer <id>" command.
			// Notes: The trailing newline is required!
			$cmd = ROUTER_CMD_IF . $selectedInterface . "\n";
			fwrite($pipes[PROCESS_PIPE_STDIN], $cmd);
			fflush($pipes[PROCESS_PIPE_STDIN]);	


			// Wait for command prompt.
			wait_for_str($pipes, TELNET_PROMPT_CMD_CONFIG_IF, 
					 PROCESS_PIPE_STDOUT); 


			// Run the command to delete old dialer string if 
			// it has been set.
			if(!empty($cur_dialer_str)) {
				// Notes: The trailing newline is required!
				$cmd = ROUTER_CMD_UNSET_IF_STR . $cur_dialer_str . "\n";
				fwrite($pipes[PROCESS_PIPE_STDIN], $cmd);
				fflush($pipes[PROCESS_PIPE_STDIN]);	


				// Wait for command prompt.
				wait_for_str($pipes, TELNET_PROMPT_CMD_CONFIG_IF, 
						 PROCESS_PIPE_STDOUT); 
			}


			// Set new dialer string.
			// Notes: The trailing newline is required!
			$cmd = ROUTER_CMD_SET_IF_STR . $new_dialer_str . "\n";
			fwrite($pipes[PROCESS_PIPE_STDIN], $cmd);
			fflush($pipes[PROCESS_PIPE_STDIN]);	


			// Wait for command prompt.
			wait_for_str($pipes, TELNET_PROMPT_CMD_CONFIG_IF, 
					 PROCESS_PIPE_STDOUT); 


			// Exit "interface dialer <id>" command.
			// Notes: The trailing newline is required!
			$cmd = TELNET_CMD_QUIT . "\n";
			fwrite($pipes[PROCESS_PIPE_STDIN], $cmd);
			fflush($pipes[PROCESS_PIPE_STDIN]);	


			// Wait for command prompt.
			wait_for_str($pipes, TELNET_PROMPT_CMD_CONFIG, 
				 PROCESS_PIPE_STDOUT); 


			// Exit "configure terminal" command.
			// Notes: The trailing newline is required!
			$cmd = TELNET_CMD_QUIT . "\n";
			fwrite($pipes[PROCESS_PIPE_STDIN], $cmd);
			fflush($pipes[PROCESS_PIPE_STDIN]);	


			// Wait for command prompt.
			wait_for_str($pipes, TELNET_PROMPT_CMD, 
				 PROCESS_PIPE_STDOUT); 

	
			// Close telnet connection.
			// Notes: The trailing newline is required!
			$cmd = TELNET_CMD_QUIT . "\n";
			fwrite($pipes[PROCESS_PIPE_STDIN], $cmd);	
			fflush($pipes[PROCESS_PIPE_STDIN]);	

		}

			
		// Close all streams before closing the child process.
		// Notes: 
		// This is very important.  If not done, proc_close
		// may run into deadlock and child process may not be
		// able to exit.  See PHP manual entry for proc_close
		// for more details.
		while(TRUE) {
			if(fclose($pipes[PROCESS_PIPE_STDIN])) {
				break;
			}
		}

		while(TRUE) {
			if(fclose($pipes[PROCESS_PIPE_STDOUT])) {
				break;
			}
		}

		while(TRUE) {
			if(fclose($pipes[PROCESS_PIPE_STDERR])) {
				break;
			}
		}


		// At the moment the exit value is 1.
		$return_value = proc_close($process);


		// Allow the telnet process to terminate properly.
		// This is to prevent a new telnet process being
		// created on subsequent attempt to read the new 
		// configurations settings before the previous 
		// telnet process has terminated properly.
		sleep(TELNET_SESSION_CLEANUP_DURATION);
	
	}


	// Unset selected checkbox.
	unset($_REQUEST["chk_ifs_index_selected"]);


	// unset session variables to force refresh of entire page,
	// which includes refreshing dialer strings.
	if(isset($_SESSION[SESS_DIALER_STRS_KEY])) {
		unset($_SESSION[SESS_DIALER_STRS_KEY]);
	}
	if(isset($_SESSION[SESS_DIALER_STRS_TIME_KEY])) {
		unset($_SESSION[SESS_DIALER_STRS_TIME_KEY]);
	}
	if(isset($_SESSION[SESS_DIALER_STRS_TIMESTAMP_KEY])) {
		unset($_SESSION[SESS_DIALER_STRS_TIMESTAMP_KEY]);
	} 

	return "";

}



/*
 * Purpose:
 * Trims the input array, which contains interfaces status data to be 
 * displayed to the user in the following way:
 *    - Remove all interfaces whose interface information is not of the
 *      format "Dialer<x>", where <x> is an integer.  (See the definition
 *      of the constant "DIALER_IF_INFO_STR_FORMAT" for more information.)
 *    - Keep only the rows to be displayed in the currently selected "page". 
 * The trimed array is returned.
 */
function trim_array($interface_status_a) {

	global $config;	

	include_once($config["library_path"] . "/database.php");


	// Error checking and special case.
	if( empty($interface_status_a) ) {
		return $interface_status_a;
	}


	// Get interface index.
	$interface_indices_a = $interface_status_a[INTERFACE_INDICES_KEY];

	// Error checking and special case.
	if( empty($interface_indices_a) ) {
		return $interface_status_a;
	}


	// Get interface description.
	$interface_desc_a = $interface_status_a[INTERFACE_DESC_KEY];

	// Get interface type.
	$interface_type_a = $interface_status_a[INTERFACE_TYPE_KEY];

	// Get interface admin status.
	$interface_admin_status_a = $interface_status_a[INTERFACE_ADMIN_STATUS_KEY];
		
	// Get interface operation status.
	$interface_op_status_a = $interface_status_a[INTERFACE_OP_STATUS_KEY];	

	// Get interface dialer ids.
	$interface_dialer_if_ids_a = $interface_status_a[INTERFACE_DIALER_IDS_KEY];

	// Get interface dialer strings.
	$interface_dialer_str_a = $interface_status_a[INTERFACE_DIALER_STRS_KEY];
	


	// Remove all elements/rows that will not be displayed. 
	$new_indices_arr1 = array();
	$index = 1;
	for($i = 1; $i <= count($interface_indices_a); $i++) {
		$if_index = $interface_indices_a[$i];
		if(!empty($interface_desc_a[$if_index])) {
			$new_indices_arr1[$index++] = $if_index; 
		}
	}


	// Should execute at most 2 iterations.
	$sqlQuery = "select value from settings where name='telix_dialer_strs_rows_per_page'";
	$res = db_fetch_row($sqlQuery);
	$telix_dialer_strs_num_rows = $res["value"];

	// If number of rows per page is not set, factory default is used.
	if( (!(isset($telix_dialer_strs_num_rows))) || (empty($telix_dialer_strs_num_rows)) ||
	    (!is_numeric($telix_dialer_strs_num_rows)) ||
          (intval($telix_dialer_strs_num_rows) < 1) ) {
		$telix_dialer_strs_num_rows = FACTORY_DEFAULT_ROWS_PER_PAGE;
	}

	$telix_dialer_strs_num_rows = intval($telix_dialer_strs_num_rows);

	while(true) {
		// Get the currently selected page.  And then work out the first element
		// of the interface index array which is to be displayed as the first row 
		// on currently selected page.  Note that the index(/key) of first element 
		// in the interface index array is 1.  Then extract the portion of the
		// array that will be needed (i.e. the rows/data to be displayed on 
		// currently selected page).
		$cur_page = $_REQUEST["page_dialer_strs"];
		$start_offset = (($cur_page-1) * $telix_dialer_strs_num_rows) + 1;
	
		$new_indices_arr2 = array();
		$new_desc_arr = array();
		$new_type_arr = array();
		$new_admin_status_arr = array();
		$new_op_status_arr = array();
		$new_dialer_if_ids_arr = array();
		$new_dialer_strs_arr = array();
		$index = 1;
		for($i = $start_offset; $i <= count($new_indices_arr1); $i++) {

			$if_index = $new_indices_arr1[$i];

			if(!empty($interface_desc_a[$if_index])) {
				// If description of this interface is not empty, we want to
				// display it.  Else ignore it.
			
				// Process interface index.
				$new_indices_arr2[$index] = $if_index;
				$index++; 

				// Process description array.
				$value = $interface_desc_a[$if_index];
				$new_desc_arr[$if_index] = $value;

				// Process type array.
				$value = $interface_type_a[$if_index];
				$new_type_arr[$if_index] = $value;

				// Process admin status array.
				$value = $interface_admin_status_a[$if_index];
				$new_admin_status_arr[$if_index] = $value;

				// Process operational status array.
				$value = $interface_op_status_a[$if_index];
				$new_op_status_arr[$if_index] = $value;

				// Process interface dialer ids array.
				$value = $interface_dialer_if_ids_a[$if_index];
				$new_dialer_if_ids_arr[$if_index] = $value;

				// Process interface dialer strings array.
				$dialer_id = $interface_dialer_if_ids_a[$if_index];

				if(isset($interface_dialer_str_a[$dialer_id])) {
					$value = $interface_dialer_str_a[$dialer_id];
					$new_dialer_strs_arr[$dialer_id] = $value;  
				} 
								
			}

		
			// We have reached the number of rows limit per page.
			if($index > $telix_dialer_strs_num_rows) {
				break;
			}
		} 

		// If there is at least a row of data to be displayed, then we are
		// done.  Otherwise, try and display rows of data on page 1.
		if(!empty($new_indices_arr2)) {
			break;
		} else {
		
			if($_REQUEST["page_dialer_strs"] == 1) {
				// This is the case where there is nothing 
				// to display on first page.
				return array(INTERFACE_INDICES_KEY=>array());
			}

			$_REQUEST["page_dialer_strs"] = 1;
			$_SESSION["sess_router_dialer_strs_current_page"] = $_REQUEST["page_dialer_strs"];
		}

	}
	
	
	// Create a new interface status array based on processing performed
	// and return it.
	$new_interface_status_a = array();
	$new_interface_status_a[INTERFACE_INDICES_KEY] = $new_indices_arr2;
	$new_interface_status_a[INTERFACE_DESC_KEY] = $new_desc_arr;
	$new_interface_status_a[INTERFACE_TYPE_KEY] = $new_type_arr;
	$new_interface_status_a[INTERFACE_ADMIN_STATUS_KEY] = $new_admin_status_arr;
	$new_interface_status_a[INTERFACE_OP_STATUS_KEY] = $new_op_status_arr;
	$new_interface_status_a[INTERFACE_DIALER_IDS_KEY] = $new_dialer_if_ids_arr;
	$new_interface_status_a[INTERFACE_DIALER_STRS_KEY] = $new_dialer_strs_arr;

	return $new_interface_status_a;

}



/*
 * Purpose: 
 * Removes status information of interfaces whose interface
 * information do not have the format 'Dialer<x>'.
 */
function trim_none_dialers($interface_status_a) {

	global $config;	

	include_once($config["library_path"] . "/database.php");


	// Error checking and special case.
	if( empty($interface_status_a) ) {
		return $interface_status_a;
	}


	// Get interface index.
	$interface_indices_a = $interface_status_a[INTERFACE_INDICES_KEY];

	// Error checking and special case.
	if( empty($interface_indices_a) ) {
		return $interface_status_a;
	}


	// Get interface description.
	$interface_desc_a = $interface_status_a[INTERFACE_DESC_KEY];

	// Get interface type.
	$interface_type_a = $interface_status_a[INTERFACE_TYPE_KEY];

	// Get interface admin status.
	$interface_admin_status_a = $interface_status_a[INTERFACE_ADMIN_STATUS_KEY];
		
	// Get interface operation status.
	$interface_op_status_a = $interface_status_a[INTERFACE_OP_STATUS_KEY];	


	// Remove all elements/rows that will not be displayed.  That is, 
      // interfaces whose interface information is not of the format
      // "Dialer<x>".
	$new_indices_arr1 = array();
	$new_desc_arr = array();
	$new_type_arr = array();
	$new_admin_status_arr = array();
	$new_op_status_arr = array();
	$new_dialer_if_id_arr = array();
	$index = 1;
	for($i = 1; $i <= count($interface_indices_a); $i++) {
		$if_index = $interface_indices_a[$i];
		if( ereg(DIALER_IF_INFO_STR_FORMAT, $interface_type_a[$if_index], $regs) ) {

			// Process interface index.
			$new_indices_arr1[$index++] = $if_index; 

			// Process description array.
			$value = $interface_desc_a[$if_index];
			$new_desc_arr[$if_index] = $value;

			// Process type array.
			$value = $interface_type_a[$if_index];
			$new_type_arr[$if_index] = $value;

			// Process admin status array.
			$value = $interface_admin_status_a[$if_index];
			$new_admin_status_arr[$if_index] = $value;

			// Process operational status array.
			$value = $interface_op_status_a[$if_index];
			$new_op_status_arr[$if_index] = $value;

			// Process interface dialer id.
			$value = $regs[REG_PATTERN_MATCH_INDEX];   // there is at most one match in the regular expression
			$new_dialer_if_id_arr[$if_index] = $value;
			
		}
	
	}	

	
	// Get dialer string for each dialer interface.
	$new_dialer_str_arr = get_dialer_strs(array_values($new_dialer_if_id_arr));


	// Extract error message, if there is one.
	$error_array = array();
	if(isset($new_dialer_str_arr[ERR_MSG_STR])) {
		$error_array[ERR_MSG_STR] = $new_dialer_str_arr[ERR_MSG_STR];
		unset($new_dialer_str_arr[ERR_MSG_STR]);	
	}


	// Error checking.
	if ( (!empty($new_indices_arr1)) && (empty($new_dialer_str_arr)) ) {

		$res_array = array(INTERFACE_INDICES_KEY=>array());
		
		if(!empty($error_array)) {
			$res_array[ERR_MSG_STR] = $error_array[ERR_MSG_STR];
		}		

		return $res_array;
	}


	// Create a new interface status array based on processing performed
	// and return it.
	$new_interface_status_a = array();
	$new_interface_status_a[INTERFACE_INDICES_KEY] = $new_indices_arr1;
	$new_interface_status_a[INTERFACE_DESC_KEY] = $new_desc_arr;
	$new_interface_status_a[INTERFACE_TYPE_KEY] = $new_type_arr;
	$new_interface_status_a[INTERFACE_ADMIN_STATUS_KEY] = $new_admin_status_arr;
	$new_interface_status_a[INTERFACE_OP_STATUS_KEY] = $new_op_status_arr;
	$new_interface_status_a[INTERFACE_DIALER_IDS_KEY] = $new_dialer_if_id_arr;
	$new_interface_status_a[INTERFACE_DIALER_STRS_KEY] = $new_dialer_str_arr;

	if(!empty($error_array)) {
		$new_interface_status_a[ERR_MSG_STR] = $error_array[ERR_MSG_STR];
	}

		
	return $new_interface_status_a;


}





/*
 * Purpose: 
 *    Get dialer string of each router interface (dialer interfaces) 
 *    whose in dialer id/number is in the input array (e.g. dialer id for
 *    Dialer56 is 56).
 *
 * Inputs:
 *    dialer_if_id_arr - array(interface_index => dialer id).
 *
 * Returns:
 *    An array of dialer strings:
 *       {dialer_id => dialer_string}.
 */
function get_dialer_strs($dialer_if_ids) {

	
	// No dialer interfaces.
	if(empty($dialer_if_ids)) {
		return array();
	}


	// Get required Telix settings.
	// Get router IP address.
	$sqlQuery = "select value from settings where name='telix_dialer_strs_router_ip'";
	$res = db_fetch_row($sqlQuery);
	$routerIP = $res["value"];

	// Get username of the telnet account setup in the router that is 
	// to be used by Telix.
	$sqlQuery = "select value from settings where name='telix_dialer_strs_telnet_username'";
	$res = db_fetch_row($sqlQuery);
	$username = $res["value"];

	// Get the corresponding password of the telnet account setup in 
	// the router that is to be used by Telix.
	$sqlQuery = "select value from settings where name='telix_dialer_strs_telnet_password'";
	$res = db_fetch_row($sqlQuery);
	$password = $res["value"];

	// Telix settings error checking.
	if( (!(isset($routerIP))) || (empty($routerIP)) ) {
		
		$err_msg = "Error: 'Router IP Address' setting for " .
			     "Router Dialer Strings Management in " .
			     "Configuration > Settings > Telix tab " .
			     "is not set.";

		return array(ERR_MSG_STR => $err_msg);

	}

	if( (!(isset($username))) || (empty($username)) ) {

		$err_msg = "Error: 'Telnet Account Username' setting " .
			     "for Router Dialer Strings Management in " .
			     "Configuration > Settings > Telix tab " .
			     "is not set.";

		return array(ERR_MSG_STR => $err_msg);

	}

	if( (!(isset($password))) || (empty($password)) ) {

		$err_msg = "Error: 'Telnet Account Password' setting " .
			     "for Router Dialer Strings Management in " .
			     "Configuration > Settings > Telix tab " .
			     "is not set.";

		return array(ERR_MSG_STR => $err_msg);

	}


	if(!isset($_REQUEST["chk_refresh_page_with_dialer_str"])) {
		
		if(isset($_SESSION[SESS_DIALER_STRS_KEY])) {
			// Already has a cached version.  
			// Use the cached version if it was obtained
			// less than DIALER_STRS_REFRESH_INTERVAL seconds
			// ago.
			$time_diff = time()-$_SESSION[SESS_DIALER_STRS_TIME_KEY];
			if($time_diff < DIALER_STRS_REFRESH_INTERVAL)   {
				return $_SESSION[SESS_DIALER_STRS_KEY];
			}
		}

	} 


	// Specify stdin, stdout and stderr to be used by the child 
	// process that will be created to run telnet.  Set all 
	// pipes to be of type "pipe".
	$descriptor_spec = 
		array(
			0 => array(PROCESS_PIPE_TYPE_PIPE, 
                             PROCESS_PIPE_MODE_READ),	// stdin
			1 => array(PROCESS_PIPE_TYPE_PIPE, 
                             PROCESS_PIPE_MODE_WRITE), // stdout		
			2 => array(PROCESS_PIPE_TYPE_PIPE, 
                             PROCESS_PIPE_MODE_WRITE)	// stderr		
		);


	// Execute the telnet command and keep track of the resource
	// returned which represents the process.
	$telnet_cmd = "telnet " . $routerIP;
	$process = proc_open($telnet_cmd, $descriptor_spec, $pipes);


	// If command is executed successfully and a representation 
	// of the process is returned.
	if(is_resource($process)) {
	
		// $pipes now looks like this:
		// 0 => writeable handle connected to child stdin.
		// 1 => readable handle connected to child stdout.
		// 2 => readable handle connected to child stderr.


		// This function currently supports regular files and 
		// socket streams only.  Therefore, it doesn't seem 
		// to make much different whether to set streams to 
		// blocking mode or non-blocking mode.  
  		// However, this function  must be called with either
		// blocking or non-blocking mode or the page doesn't get
		// loaded within expected time (not sure it will ever be
		// loaded though if the function is not called.
		// Ideally the streams (stdout and stderr, possibly stdin) 
		// should be set to blocking mode so that when the function
		// fgets is called, it will block until a non-empty line
		// of string is read.  Otherwise a loop is needed to 
		// continously calling fgets until a non-empty string is 
		// read, which is the current behavior of Telix.  There
		// doesn't seem to be a way around that.
		// Also note that PHP manual (see entry on Process Control
		// Functions) specifies that "Process Control functions 
		// should not be enabled within a webserver environment
		// and unexpected results may happen if any Process 
		// Control functions are used within a webserver
		// environment".  Therefore, Process Control functions 
		// can't be used to creat blocking behavior to make 
		// Telix more efficient.  Also note that it also makes
		// the program much more complex.
		stream_set_blocking($pipes[PROCESS_PIPE_STDIN], 
				 	  STREAM_NON_BLOCKING);
		stream_set_blocking($pipes[PROCESS_PIPE_STDOUT], 
					  STREAM_NON_BLOCKING);
		stream_set_blocking($pipes[PROCESS_PIPE_STDERR], 
					  STREAM_NON_BLOCKING);


		// Login.  The number of attempts is 1 less than the
		// total number allowed so that the connection is
		// not automatically closed by the other side. 
		for($i = 0; $i <= TELNET_LOGIN_ATTEMPTS; $i++) {

			// Wait username prompt and then enter the username.
			wait_for_str($pipes, TELNET_PROMPT_USERNAME, 
					 PROCESS_PIPE_STDOUT);
			$telnet_username = $username . "\n";
			fwrite($pipes[PROCESS_PIPE_STDIN], $telnet_username);	
			fflush($pipes[PROCESS_PIPE_STDIN]);
		

			// Wait for Password prompt and then enter the password.
			wait_for_str($pipes, TELNET_PROMPT_PASSWORD, 
					 PROCESS_PIPE_STDOUT);
			$telnet_password = $password . "\n";
			fwrite($pipes[PROCESS_PIPE_STDIN], $telnet_password);	
			fflush($pipes[PROCESS_PIPE_STDIN]);


			// Wait for command prompt.
			$isLoggedIn = wait_for_str_err(
						$pipes, TELNET_PROMPT_CMD, 
				  		PROCESS_PIPE_STDOUT, 
						TELNET_PROMPT_AUTHENTICATION_FAILED);

			if($isLoggedIn) {
				break;
			}

		}		


		// If logged in successfully, run the command to extract 
		// dialer strings of each dialer interface.
		if($isLoggedIn) {

			$dialer_strs_arr = array();				
			foreach($dialer_if_ids as $id) {

				// Send the command to extract configuration details
				// using the current dialer interface id.
				// Note: Trailing newline is required!
				$cmd = ROUTER_CMD_SHOW_IF . $id . "\n";
				fwrite($pipes[PROCESS_PIPE_STDIN],  $cmd);	
				fflush($pipes[PROCESS_PIPE_STDIN]);


				// Read the output of the command.
				// Very light parsing.
				$has_started = FALSE;				
				while(TRUE) {

					// Read next line.
					$line = fgets($pipes[PROCESS_PIPE_STDOUT]);
					$line = trim($line);


					// This is to remove the junk at the front of 
					// the line which immediately follows the line
					// "--More--" which waits for space character
					// to continue display the output.
					if(isset($has_read_more_delimiter)) {
				
						// We have seen the "--More--" line.
						if(!empty($line)) {
				
							for($i = 0; $i < strlen($line); $i++) {
								// Found the first character 
								// which is not a junk.  
								// Extract the remaining string.
								if( ctype_alnum($line[$i]) ) {
									$line = substr($line, $i);
									break;
								}
							}

							unset($has_read_more_delimiter);

						}
		
					}


					// Parse each line.
					if(strcmp($line, ROUTER_START_CONFIG_MARKER) == 
			      	   STR_COMPARE_EQUALS) {
 
						// Parsed the marker which indicates the 
 						// beginning of the configuration settings 
						// for the interface.
						$has_started = TRUE;				

					} elseif(strcmp($line, ROUTER_END_CONFIG_MARKER) == 
			      	         STR_COMPARE_EQUALS) {

						// Parsed the marker which indicates the end
						// of the configuration settings for the 
						// interface.					
						break;

					} elseif(strcmp($line, TELNET_PROMPT_MORE) == 
						   STR_COMPARE_EQUALS) {
						// Telnet only returned a portion of the output of the 
						// command executed.  Send a space character to tell
						// telnet to continue to display the output.

						// Send space character to display remaining entries
						// of this interface.
						fwrite($pipes[PROCESS_PIPE_STDIN], TELNET_CMD_SPACE);	
						fflush($pipes[PROCESS_PIPE_STDIN]);
						$has_read_more_delimiter = TRUE;				

					} elseif(ereg(ROUTER_DIALER_STR_FORMAT, $line, $regs)) {
						// Found the dialer string which is what we want.

						// Dialer string found in the configuration entry
						// for the interface.
						if($has_started) {					
							$dialer_strs_arr[$id] = $regs[REG_PATTERN_MATCH_INDEX];		
						}
					
					} 

				}

				if( !isset($dialer_strs_arr[$id]) ) {
					$dialer_strs_arr[$id] = ROUTER_ERR_DIAL_STR_NOT_SET;
				}

				// Wait for command prompt.
				wait_for_str($pipes, TELNET_PROMPT_CMD, 
						 PROCESS_PIPE_STDOUT);

			}   // end foreach
		
		
			// Already read the command prompt.
			// Close telnet connection.
			// Notes: The trailing newline is required!
			$cmd = TELNET_CMD_QUIT + "\n ";
			fwrite($pipes[PROCESS_PIPE_STDIN], $cmd);	
			fflush($pipes[PROCESS_PIPE_STDIN]);		


			// The telnet process occassionally lock up/hang when
			// pages are refreshed at very short interval.  Not exactly
			// certain what the cause is. Usually, by killing the telnet
			// process manually using "kill" command, the page will
			// stop refreshing and loaded with expected contents.  This
			// suggests that the deadlock occurred probably after the
			// exit command was sent or else the page should not have
			// the expected contents.  It is possible that telnet is
			// trying to write something to output buffer(s) which is
			// full and thus it continuously polling the buffer to see
			// if there is any free space.  This leads to the observation
			// that when "ps -A | grep telnet" command is executed 
			// several times, the CPU time consumed by telnet continues
			// to increase.  
			// It appears that telnet outputs the value '0' 
			// (or is it '\0', i.e., ASCII 0?).  So created a new 
			// function "wait_for_str_no_trim" to consume the final 
			// telnet output.  It is similar to "wait_for_str" except 
			// that it does not trim each line read (using fgets) 
			// and the string to match is defined by the constant
			// TELNET_SESSION_END_STR.  
			// Note that this constant, i.e., the final telnet output
			// might be implementation/platform dependent.  This is
			// tested using the environment in CactiEZ cd image which can
			// be downloaded from http://cactiusers.org/. 
			// 
			// 
			// 					
			wait_for_str_no_trim($pipes, TELNET_SESSION_END_STR, 
				    	         PROCESS_PIPE_STDOUT);

		}



		// Close all streams before closing the child process.
		// Notes: 
		// This is very important.  If not done, proc_close
		// may run into deadlock and child process may not be
		// able to exit.  See PHP manual entry for proc_close
		// for more details.
		while(TRUE) {
			if(fclose($pipes[PROCESS_PIPE_STDIN])) {
				break;
			}
		}

		while(TRUE) {
			if(fclose($pipes[PROCESS_PIPE_STDOUT])) {
				break;
			}
		}

		while(TRUE) {
			if(fclose($pipes[PROCESS_PIPE_STDERR])) {
				break;
			}
		}


		// At the moment the exit value is 1.
		$return_value = proc_close($process);


		// Allow the telnet process to terminate properly.
		// This is to prevent deadlock on subsequent 
		// attempt to read the new configurations settings.
		sleep(TELNET_SESSION_CLEANUP_DURATION);


		if($isLoggedIn) {
			
			// Cache dialer strings extracted and store the 
			// current time so we know when the values were
			// last refreshed.
			$_SESSION[SESS_DIALER_STRS_KEY] = $dialer_strs_arr;
			$cur_time = time();
			$_SESSION[SESS_DIALER_STRS_TIME_KEY] = $cur_time;
			$cur_timestamp = date(CUR_TIMESTAMP_FORMAT, $cur_time);
			$_SESSION[SESS_DIALER_STRS_TIMESTAMP_KEY] = 
				$cur_timestamp;	
				
			return $dialer_strs_arr;
		} else {

			$err_msg = "Error: Telnet login failed.  Please " .
				     "check 'Telnet Account Username' and " .
			    	     "'Telnet Account Password' settings in " .
				     "Configuration > Settings > Telix tab.";

			return array(ERR_MSG_STR => $err_msg);

		}

	}   // end if 


	// Unable to start a telnet process.
	$err_msg = "Error: Unable to establish Telnet connection.  " .
		     "Telnet process creation failed.";

	return array(ERR_MSG_STR => $err_msg);

}   // end function get_dialer_strs



/*
 * Purpose: 
 *    Wait for command prompt to be written to standard output
 *    by the telnet process.
 *
 * Inputs:
 *    - $pipes
 *         Reference to the array of file handles, which contains
 *         the stdin, stdout and stderr file handles.
 *    - $str_to_wait
 *         Text string that we are expecting.
 *
 *    - $index
 *         Array index used to access $pipes, i.e., which file handle
 *         in $pipes to use.
 *
 * Notes:
 *    - Ideally, the streams stdout and stderr (possibly stdin) 
 *      should be set to block using stream_set_blocking function
 *      so that fgets blocks until a non-empty line of text is read.
 *      However, stream_set_blocking currently supports regular files
 *      and socket streams only.  Therefore, CPU resources is wasted
 *      on executing fgets in the loop, and it seems like there is no
 *      way around that.
 *      Also, when calling stream_set_blocking with non-blocking mode
 *      (see functions get_dialer_strs and form_action_set_dialer_string),
 *      the number of loops executed on average seems to be less than 
 *      calling it with blocking mode.  Since calling that function with
 *      blocking mode has no effect, the stream_set_blocking function in 
 *      the two functions mentioned before is called with non-blocking 
 *      mode.
 */
function wait_for_str(&$pipes, $str_to_wait, $index) {

	// Continuing read the stdout pipe until telnet process 
	// returned a particular line of text (e.g. command prompt) so 
	// that a particular action can then be take (e.g. send a 
	// command).
	while(TRUE) {

		// Read the next line in stdout.
		// Removes leading and trailing whitespaces.
		$line = fgets($pipes[$index]);   
		$line = trim($line);   

		if(strcmp($line, $str_to_wait) == STR_COMPARE_EQUALS) {
			// Read the line of text we are waiting for.
			// Exit the loop.	
			break;

		} 

	}

	return;

}


/*
 * Purpose: 
 *    Wait for command prompt to be written to standard output
 *    by the telnet process.
 *
 * Inputs:
 *    - $pipes
 *         Reference to the array of file handles, which contains
 *         the stdin, stdout and stderr file handles.
 *    - $str_to_wait
 *         Text string that we are expecting.
 *
 *    - $index
 *         Array index used to access $pipes, i.e., which file handle
 *         in $pipes to use.
 *
 *    - $err_str 
 *         If this string is read before $str_to_wait, return FALSE.
 *
 *
 * Returns:
 *    True if $str_to_wait is read before $err_str.
 *
 * Notes:
 * - Ideally, the streams stdout and stderr (possibly stdin) 
 *   should be set to block using stream_set_blocking function
 *   so that fgets blocks until a non-empty line of text is read.
 *   However, stream_set_blocking currently supports regular files
 *   and socket streams only.  Therefore, CPU resources is wasted
 *   on executing fgets in the loop, and it seems like there is no
 *   way around that.
 *   Also, when calling stream_set_blocking with non-blocking mode
 *   (see functions get_dialer_strs and form_action_set_dialer_string),
 *   the number of loops executed on average seems to be less than 
 *   calling it with blocking mode.  Since calling that function with
 *   blocking mode has no effect, the stream_set_blocking function in 
 *   the two functions mentioned before is called with non-blocking 
 *   mode.
 */
function wait_for_str_err(&$pipes, $str_to_wait, $index, $err_str) {

	// Continuing read the stdout pipe until telnet process 
	// returned a particular line of text (e.g. command prompt) so 
	// that a particular action can then be take (e.g. send a 
	// command).
	while(TRUE) {

		// Read the next line in stdout.
		// Removes leading and trailing whitespaces.
		$line = fgets($pipes[$index]);   
		$line = trim($line);   

		if(strcmp($line, $err_str) == STR_COMPARE_EQUALS) {
			// Read the line of text that is considered as
			// exception.
		
			return FALSE;

		} 

		if(strcmp($line, $str_to_wait) == STR_COMPARE_EQUALS) {
			// Read the line of text we are waiting for.
			// Exit the loop.	
			break;

		} 

	}

	return TRUE;

}


/*
 * Purpose: 
 * Wait for command prompt to be written to standard output
 * by the telnet process.
 *
 * Inputs:
 *    - $pipes
 *         Reference to the array of file handles, which contains
 *         the stdin, stdout and stderr file handles.
 *    - $str_to_wait
 *         Text string that we are expecting.
 *
 *    - $index
 *         Array index used to access $pipes, i.e., which file handle
 *         in $pipes to use.
 *
 *    - $err_str 
 *         If this string is read before $str_to_wait, return FALSE.
 *
 * 
 */
function wait_for_str_no_trim(&$pipes, $str_to_wait, $index) {

	// Continuing read the stdout pipe until 0 or
	// ASCII 0 is read.
	while(TRUE) {

		// Read the next line in stdout.
		// Does not use "trim" function as it will remove "0"/
		// ASCII 0.
		$line = fgets($pipes[$index]);   

		if(strcmp($line, $str_to_wait) == STR_COMPARE_EQUALS) {
			// Read the line of text we are waiting for.
			// Exit the loop.	
			break;

		} 

	}

	return;

}




?>


































