<?php
/*******************************************************************************

    Program: Telix.

*******************************************************************************/


// Constants

// ---
// Defines maximum number of pages to be displayed when showing all router 
// interfaces.
define("MAX_DISPLAY_PAGES", 21);


// ---
// Keys to index and extract interface status information.
define("INTERFACE_INDICES_KEY", "interface_indices");
define("INTERFACE_DESC_KEY", "interface_desc");
define("INTERFACE_TYPE_KEY", "interface_type");
define("INTERFACE_ADMIN_STATUS_KEY", "interface_admin_status");
define("INTERFACE_OP_STATUS_KEY", "interface_op_status");
define("INTERFACE_ERR_STR", "interface_err_string");


// ---
// Define constants related to SNMP function calls.
define("SNMPv1", 0);
define("SNMPv2c", 1);
define("SNMPv3", 2);
define("DELIMITER_OFFSET", 2);   // for ": " string in snmpwalk output.


//---
// Error Messages..
define("ERR_MSG_STR", "err_msg");
define("ERR_NO_SELECTION", "Error: Did not select an interface to update");


// ---
// Note that this value is used only when the user did not
// set the default page refresh rate (for View Router Interfaces Status)
// through Configuration > Settings > Telix tab.page.
define("FACTORY_DEFAULT_IFS_VIEW_REFRESH_RATE", 300);


// ---
// Factory default of the number of rows displayed on this page.
define("FACTORY_DEFAULT_IFS_VIEW_ROWS_PER_PAGE", 15);



// Change current working directory to Cacti root directory.  This is
// required to allow existing Cacti PHP files like auth.php etc. to be used
// in plugin code, as include statements in existing Cacti PHP files assumes
// that current working directory is the Cacti root directory (i.e. relative
// paths (relative to Cacti root directory) are used in include statements).  
// This also enables reuse of existing Cacti code with minimal changes.
chdir('../../');


// Authenticate the user.  Also includes ./include/config.php.
include_once("./include/auth.php");



// This is a read-only view of interfaces information/status and 
// therefore the processing involved is just to display this page.
// Set default action.
if (!isset($_REQUEST["action"])) { $_REQUEST["action"] = ""; }

switch ($_REQUEST["action"]) {

	default:
		$refresh = get_refresh_interval();
		set_session_variables();
		$interface_status = get_interface_status();
		$interface_status_trimed = trim_array($interface_status);		
		include_once("./include/top_header.php");
		display_telix_ifs_view_page($interface_status, $interface_status_trimed);
		include_once("./include/bottom_footer.php");
		break;
}


/*
 * Purpose: 
 *    Setup session variables and destroy those used by other Telix 
 *    PHP pages which conflicts with session variables used by 
 *    this page.
 */
function set_session_variables() {

	// Setup session variables.
	// Remember which page is being displayed.
	if(isset($_SESSION["sess_router_ifs_current_page"])) {
		unset($_SESSION["sess_router_ifs_current_page"]);
	}	
	if(isset($_SESSION["sess_router_dialer_strs_current_page"])) {
		unset($_SESSION["sess_router_dialer_strs_current_page"]);
	}
	if(isset($_SESSION["dialer_strs"])) {
		// See "set_session_variables" function in 
		// telix_dialer_strs.php.
		unset($_SESSION["dialer_strs"]);
	}
	if(isset($_SESSION["dialer_strs_time"])) {
		// See "set_session_variables" function in 
		// telix_dialer_strs.php.
		unset($_SESSION["dialer_strs_time"]);
	}
	if(isset($_SESSION["dialer_strs_timestamp"])) {
		// See "set_session_variables" function in 
		// telix_dialer_strs.php.
		unset($_SESSION["dialer_strs_timestamp"]);
	}
	if(isset($_REQUEST["page_ifs"])) {
		unset($_REQUEST["page_ifs"]);
	}
	if(isset($_REQUEST["page_dialer_strs"])) {
		unset($_REQUEST["page_dialer_strs"]);
	}	
	load_current_session_value("page_ifs_view", "sess_router_ifs_view_current_page", "1");
	load_current_session_value("page_refresh_rate", "sess_ifs_view_page_refresh_rate", "0");

}





/*
 * Purpose: 
 *    Displays the Telix Viwe Router Interfaces Status page.
 */
function display_telix_ifs_view_page($interface_status, $interface_status_trimed) {

	global $config, $colors;

	include_once($config["library_path"] . "/database.php");


	// Display error message if it is set.
	if(isset($interface_status[ERR_MSG_STR])) {
		$err_msg =  "<tr><td> " .
				"<font color='red'> <b> " .
	      	      $interface_status[ERR_MSG_STR] . "</b> </font><br>" .
				"</td></tr>";

		html_start_box("", "98%", "FFFFFF", "3", "center", "");
		print $err_msg;
		html_end_box();
	}

	echo "<br><br>";



	// Display task selection form.
	html_start_box("<strong>Tasks &nbsp&nbsp&nbsp&nbsp[task: View Router Interfaces Status]</strong>", "98%", $colors["header"], "3", "center", "");

	include($config["base_path"] . "/plugins/telix/telix_select_task_form_ifs_view.php");  

	html_end_box();


	// Display the section that allows user to choose the page refresh rate 
  	// and to invoke immediate manual refresh.
	html_start_box("<strong>Router Interfaces</strong>", "98%", $colors["header"], "3", "center", "");

	include("./plugins/telix/telix_refresh_form_ifs_view.php");  

	html_end_box();


	// Display the section that shows the details of the router interfaces.
	html_start_box("", "98%", $colors["header"], "3", "center", "");


	// Count the number of rows to display.  
	//Rows with description "" are not displayed.
	$num_interfaces = count($interface_status[INTERFACE_INDICES_KEY]);


	// If number of interfaces found is greater than 0, then remove those
  	// (from the count) whose description is "" (since they are not to be
  	// displayed.
	if($num_interfaces > 0) {
		$ifs_desc_a = $interface_status[INTERFACE_DESC_KEY];

		foreach($ifs_desc_a as $key => $value) {

			if(empty($value)) {
				$num_interfaces--;
			}
		}
	}


	// generate page list
	$sqlQuery = "select value from settings where name='telix_ifs_view_rows_per_page'";
	$res = db_fetch_row($sqlQuery);
	$telix_ifs_view_num_rows= $res["value"];

	if( (!(isset($telix_ifs_view_num_rows))) || (empty($telix_ifs_view_num_rows)) ||
          (!is_numeric($telix_ifs_view_num_rows)) || 
	    (intval($telix_ifs_view_num_rows) < 1) ) {
		$telix_ifs_view_num_rows = FACTORY_DEFAULT_IFS_VIEW_ROWS_PER_PAGE;
	}

	$telix_ifs_view_num_rows = intval($telix_ifs_view_num_rows);

	$url_page_select = get_page_list($_REQUEST["page_ifs_view"], MAX_DISPLAY_PAGES, 
						   $telix_ifs_view_num_rows, $num_interfaces, "");

	
	// Generate and display the navigation part of the output.
	// The colspan here must match number of columns as defined in $display_text
	$nav = "<tr bgcolor='#" . $colors["header"] . "'>			
			<td colspan='6'>
				<table width='100%' cellspacing='0' cellpadding='0' border='0'>
					<tr>
						<td align='left' class='textHeaderDark'>
							<strong>&lt;&lt; "; if ($_REQUEST["page_ifs_view"] > 1) { $nav .= "<a class='linkOverDark' href='telix_ifs_view.php?page_ifs_view=" . ($_REQUEST["page_ifs_view"]-1) . "'>"; } $nav .= "Previous"; if ($_REQUEST["page_ifs_view"] > 1) { $nav .= "</a>"; } $nav .= "</strong>
						</td>\n
						<td align='center' class='textHeaderDark'>
							Showing Rows " . (($num_interfaces>0)?(($telix_ifs_view_num_rows*($_REQUEST["page_ifs_view"]-1))+1): 0) . " to " . ((($num_interfaces < $telix_ifs_view_num_rows) || ($num_interfaces < ($telix_ifs_view_num_rows*$_REQUEST["page_ifs_view"]))) ? $num_interfaces : ($telix_ifs_view_num_rows*$_REQUEST["page_ifs_view"])) . " of $num_interfaces [$url_page_select]
												
						</td>\n
						<td align='right' class='textHeaderDark'>
							<strong>"; if (($_REQUEST["page_ifs_view"] * $telix_ifs_view_num_rows) < $num_interfaces) { $nav .= "<a class='linkOverDark' href='telix_ifs_view.php?page_ifs_view=" . ($_REQUEST["page_ifs_view"]+1) . "'>"; } $nav .= "Next"; if (($_REQUEST["page_ifs_view"] * $telix_ifs_view_num_rows) < $num_interfaces) { $nav .= "</a>"; } $nav .= " &gt;&gt;</strong>
						</td>\n
						<td align='right' class='textHeaderDark'>
						</td>\n	
					</tr>
				</table>
			</td>
		</tr>\n";

	print $nav;


	// Currently does not support sort by column.
	$display_text = array(
		"Index",
		"Description",
		"Type",
	      "Admin Status",
            "Operational Status",
		"");


	html_header($display_text);
	echo "<form action='telix_ifs_view.php' method='post'>";


	// Get all status arrays.
	if($num_interfaces > 0) {
		$ifs_index_a = $interface_status_trimed[INTERFACE_INDICES_KEY];
		$ifs_desc_a = $interface_status_trimed[INTERFACE_DESC_KEY];
		$ifs_type_a = $interface_status_trimed[INTERFACE_TYPE_KEY];
		$ifs_admin_status_a = $interface_status_trimed[INTERFACE_ADMIN_STATUS_KEY];
		$ifs_op_status_a = $interface_status_trimed[INTERFACE_OP_STATUS_KEY];
	} else {
		$ifs_index_a = array();
	}
	

	$i = 0;
	if (count($ifs_index_a) > 0) {
		foreach ($ifs_index_a as $key => $ifs_index) {

			if(!empty($ifs_desc_a[$ifs_index])) {
				form_alternate_row_color($colors["alternate"],$colors["light"],$i); $i++;
					?>
					<td>
						<?php print "$ifs_index"; ?>	
					</td>
					<td>
						<?php print "$ifs_desc_a[$ifs_index]"; ?>	
					</td>
					<td>
						<?php print "$ifs_type_a[$ifs_index]"; ?>	
					</td>
					<td>
						<?php print status_to_str($ifs_admin_status_a[$ifs_index]); ?>	
					</td>
					<td colspan=2>
						<?php print status_to_str($ifs_op_status_a[$ifs_index]); ?>	
					</td>
				</tr>
			<?php
			}
		}

		/* put the nav bar on the bottom as well */
		print $nav;
	}else{		

		$err_str =
			"<tr><td><em>No Router Interfaces Found." .
			"</em></td></tr>";
		
		print $err_str;


		// Clear session variables related to displaying page so that when data becomes
		// available, the first page is displayed.
		if(isset($_SESSION["sess_router_ifs_view_current_page"])) {
			unset($_SESSION["sess_router_ifs_view_current_page"]);
		}
		if(isset($_REQUEST["page_ifs_view"])) {
			unset($_REQUEST["page_ifs_view"]);
		}

	}

	html_end_box(false);


	print "</form>\n";
}




/*
 * Purpose: Obtain interface status using SNMP.
 * 
 * Returns:
 * Status of all interfaces on a router stored in the following format:
 * $interface_status = 
 *    array( 
 *       INTERFACE_INDICES_KEY => $interface_indices_a,
 *       INTERFACE_DESC_KEY => $interface_desc_a,
 *       INTERFACE_TYPE_KEY => $interafce_type_a,
 *       INTERFACE_ADMIN_STATUS_KEY => $interface_admin_status_a,
 *       INTERFACE_OP_STATUS_KEY => $interface_op_status_a);
 *     
 *
 * Notes:
 * - <TODO> 
 *    - Only SNMP v1 is currently covered.
 *
 * - This function is coded to a specific OID tree and thus can be
 *   considered as hardcoded.  The reason is that the PHP SNMP function
 *   snmpwalk returns an array of the following type:
 *   array = ("<type1>: <value1>", "<type2>: <value2>", ..., "<typeN>: <valueN>")
 *
 *   where <type1> = STRING, INTEGER etc.
 *   That is, it doesn't indicate which MIB object <value1> belongs to.
 *   Net-SNMP was used to determine what each value is.  For example, first MIB
 *   obejct is "ifNumber" which indicates the number of interface and second "set" of
 *   MIB objects is "ifIndex" of each interface available.
 *   As the result, the function is effectively hardcoded based on assumed data structure
 *   of the return value.
 *
 *   Also note that snmpwalk is used to take a snapshot of the state of all interfaces, 
 *   rather than using snmpget to extract state of each interface one at a time.  This
 *   is attempt to preserve the integrity of data.   
 */
function get_interface_status() {

	global $config;	

	$interface_status = array();


	include_once($config["library_path"] . "/database.php");

	// Get SNMP Version, RO Community String and Router IP Address.
	$sqlQuery = "select value from settings where name='telix_ifs_view_snmp_version'";
	$res = db_fetch_row($sqlQuery);
	$snmpVersion = $res["value"];

	$sqlQuery = "select value from settings where name='telix_ifs_view_ro'";
	$res = db_fetch_row($sqlQuery);
	$roString = $res["value"];

	$sqlQuery = "select value from settings where name='telix_ifs_view_router_ip'";
	$res = db_fetch_row($sqlQuery);
	$routerIP = $res["value"];

	$sqlQuery = "select value from settings where name='telix_ifs_view_oid'";
	$res = db_fetch_row($sqlQuery);
	$oid = $res["value"];

	$sqlQuery = "select value from settings where name='telix_ifs_view_if_desc_oid'";
	$res = db_fetch_row($sqlQuery);
	$if_desc_oid = $res["value"];


	if( !(isset($snmpVersion)) ) {
		$res_array = array(INTERFACE_INDICES_KEY=>array());
		$res_array[ERR_MSG_STR] = 
			"Error: 'SNMP Version' setting for View Router " . 
			"Interfaces Status in " . 
			"Configuration > Settings > Telix tab is " .
			"not set.";

		return $res_array;
	}

	if( (!(isset($roString))) || (empty($roString)) ) {
		$res_array = array(INTERFACE_INDICES_KEY=>array());
		$res_array[ERR_MSG_STR] = 
			"Error: 'RO Community String' setting for View " .
			"Router Interfaces Status in " .
			"Configuration > Settings > Telix " .
			"tab is not set.";

		return $res_array;
	}

	if( (!(isset($routerIP))) || (empty($routerIP)) ) {
		$res_array = array(INTERFACE_INDICES_KEY=>array());
		$res_array[ERR_MSG_STR] = 
			"Error: 'Router IP Address' setting for View " .
			"Router Interfaces Status in " .
			"Configuration > Settings > Telix tab " .
			"is not set.";

		return $res_array;
	}

	if( (!(isset($oid))) || (empty($oid)) ) {
		$res_array = array(INTERFACE_INDICES_KEY=>array());
		$res_array[ERR_MSG_STR] = 
			"Error: 'Interfaces OID' setting for View " .
			"Router Interfaces Status in " .
			"Configuration > Settings > Telix " .
			"tab is not set.";

		return $res_array;
	}

	if( (!(isset($if_desc_oid))) || (empty($if_desc_oid)) ) {
		$res_array = array(INTERFACE_INDICES_KEY=>array());
		$res_array[ERR_MSG_STR] = 
			"Error: 'Interfaces Description OID' setting for " .
			"View Router Interfaces Status in " .
			"Configuration > Settings > Telix " .
			"tab is not set.";

		return $res_array;
	}


	if($snmpVersion == SNMPv1) {
		// Also note that snmpwalk function takes two more optional parameters
  		// at the end, namely the timeout and retries which are not used at the
		// moment.
		// The '@' sign is to suppress any error or warnings being displayed to 
		// the page.  The program should handle it instead.
		$interfaceData = @snmpwalk($routerIP, $roString, $oid);

		if($interfaceData == FALSE) {
			$res_array = array(INTERFACE_INDICES_KEY=>array());
			$res_array[ERR_MSG_STR] = 
				"Error: 'snmpwalk' function call to extract " .
				"interfaces information failed.  Please try " .
				"again.  If this function call continues to " .
				"fail, please check " .
				"View Router Interfaces Status settings in " .
				"Configuration > Settings > Telix tab.";

			return $res_array;
		}

		// Get interfaces description.
		$interfaceDesc = @snmpwalk($routerIP, $roString, $if_desc_oid);	

		if($interfaceDesc== FALSE) {
			$res_array = array(INTERFACE_INDICES_KEY=>array());
			$res_array[ERR_MSG_STR] = 
				"Error: 'snmpwalk' function call to extract " .
				"interfaces description failed.  Please try " .
				"again.  If this function call continues to " .
				"fail, please check " .
				"View Router Interfaces Status settings in " .
				"Configuration > Settings > Telix tab.";

			return $res_array;
		}

	} elseif ($snmpVersion == SNMPv2c) {
		// TODO
	
	} elseif ($snmpVersion == SNMPv3) {
		// TODO

	}


	// Get the number of interfaces.
	$str = $interfaceData[0];
	$numInterfaces = (int) substr(strrchr($str, ":"), DELIMITER_OFFSET);


	// Setup offset values.  This is based on the OID tree.
	$ifIndexOffset = 1;
	$ifTypeOffset= 2;
	$ifAdminStatusOffset = 7;
	$ifOperStatusPffset = 8;


	// Get interface index.
	$startPos = $ifIndexOffset;
	$interface_indices_a = array();
	for($i=$startPos; $i <= $numInterfaces; $i++) {
		$str = $interfaceData[$startPos+($i-1)];		
		$interface_indices_a[$i] = substr(strrchr($str, ":"), DELIMITER_OFFSET);
	}
	$interface_status[INTERFACE_INDICES_KEY] = $interface_indices_a;


	// Get interface type.
	$startPos = (($ifTypeOffset-1)* $numInterfaces) + 1;
	$interface_type_a = array();
	for($i=1; $i <= $numInterfaces; $i++) {
		$str = $interfaceData[$startPos+($i-1)];		
		$str = substr(strrchr($str, ":"), DELIMITER_OFFSET);
		$interface_type_a{$interface_indices_a[$i]} = $str;
	}
	$interface_status[INTERFACE_TYPE_KEY] = $interface_type_a;



	// Get interface admin status.
	$startPos = (($ifAdminStatusOffset-1) * $numInterfaces) + 1;
	$interface_admin_status_a = array();
	for($i=1; $i <= $numInterfaces; $i++) {
		$str = $interfaceData[$startPos+($i-1)]; // str = "up(1)" or "up(2)".		
		$str = substr(strrchr($str, "("), 1, 1); 
		$interface_admin_status_a{$interface_indices_a[$i]} = $str;
	}
	$interface_status[INTERFACE_ADMIN_STATUS_KEY] = $interface_admin_status_a;
	

	
	// Get interface operation status.
	$startPos = (($ifOperStatusPffset-1)* $numInterfaces) + 1;
	$interface_op_status_a = array();
	for($i=1; $i <= $numInterfaces; $i++) {
		$str = $interfaceData[$startPos+($i-1)]; // str = "up(1)" or "up(2)".		
		$str = substr(strrchr($str, "("), 1, 1); 
		$interface_op_status_a{$interface_indices_a[$i]} = $str;
	}
	$interface_status[INTERFACE_OP_STATUS_KEY] = $interface_op_status_a;



	// Get interface description.
	$interface_desc_a = array();
	for($i=0; $i < $numInterfaces; $i++) {
		$str = $interfaceDesc [$i];		
		$interface_desc_a{$interface_indices_a[$i+1]} = substr(strrchr($str, ":"), DELIMITER_OFFSET);
	}
	$interface_status[INTERFACE_DESC_KEY] = $interface_desc_a;


	return $interface_status;


}



/*
 * Purpose: 
 *    Coverts interface status (admin or op) from numerical 
 *    representation to string (texual) representation.
 *
 * Inputs:
 *    $status - interface status (admin or op).
 *
 * Returns:
 *    String (texual) representation of the input interface
 *    status.
 */
function status_to_str($status) {
	
	$status_str = "Unknown";

	if($status == 1) {
		$status_str = '<font color="green">Up</font>';
	} elseif($status == 2) {
		$status_str = '<font color="red">Down</font>';
	} elseif($status == 3) {
		$status_str = '<font color="#ED7600">Testing</font>';
	} elseif($status == 4) {
		$status_str = '<font color="red">Unknown</font>';
	} elseif($status == 5) {
		$status_str = '<font color="#ED7600">Dormant</font>';
	} elseif($status == 6) {
		$status_str = '<font color="red">Not Present</font>';
	} elseif($status == 7) {
		$status_str = '<font color="red">Lower Layer Down</font>';
	}

	return $status_str;

}


/*
 * Purpose:
 *    Trims the input array, which contains data to be displayed to the user,
 *    so that only the rows of the currently selected "page" is included in 
 *    the array.  The trimed array is returned.
 */
function trim_array($interface_status_a) {

	global $config;

	include_once($config["library_path"] . "/database.php");


	// Error checking and special case.
	if( empty($interface_status_a) ) {
		return $interface_status_a;
	}


	// Get interface index.
	$interface_indices_a = $interface_status_a[INTERFACE_INDICES_KEY];
	
	// Error checking and special case.
	if( empty($interface_indices_a) ) {
		return $interface_status_a;
	}


	// Get interface description.
	$interface_desc_a = $interface_status_a[INTERFACE_DESC_KEY];

	// Get interface type.
	$interface_type_a = $interface_status_a[INTERFACE_TYPE_KEY];

	// Get interface admin status.
	$interface_admin_status_a = $interface_status_a[INTERFACE_ADMIN_STATUS_KEY];
		
	// Get interface operation status.
	$interface_op_status_a = $interface_status_a[INTERFACE_OP_STATUS_KEY];	


	// Remove all elements/rows that will not be displayed.
	$new_indices_arr1 = array();
	$index = 1;
	for($i = 1; $i <= count($interface_indices_a); $i++) {
		$if_index = $interface_indices_a[$i];
		if(!empty($interface_desc_a[$if_index])) {
			$new_indices_arr1[$index++] = $if_index; 
		}
	}


	//Should execute at most 2 iterations.	
	$sqlQuery = "select value from settings where name='telix_ifs_view_rows_per_page'";
	$res = db_fetch_row($sqlQuery);
	$telix_ifs_view_num_rows= $res["value"];

	if( (!(isset($telix_ifs_view_num_rows))) || (empty($telix_ifs_view_num_rows)) ||
          (!is_numeric($telix_ifs_view_num_rows)) || 
	    (intval($telix_ifs_view_num_rows) < 1) ) {
		$telix_ifs_view_num_rows = FACTORY_DEFAULT_IFS_VIEW_ROWS_PER_PAGE;
	}

	$telix_ifs_view_num_rows = intval($telix_ifs_view_num_rows);


	while(true) {
		// Get the currently selected page.  And then work out the first element
		// of the interface index array which is to be displayed as the first row 
		// on currently selected page.  Note that the index(/key) of first element 
		// in the interface index array is 1.  Then extract the portion of the
		// array that will be needed (i.e. the rows/data to be displayed on 
		// currently selected page).
		$cur_page = $_REQUEST["page_ifs_view"];
		$start_offset = (($cur_page-1) * $telix_ifs_view_num_rows) + 1;
		$new_indices_arr2 = array();
		$index = 1;
		for($i = $start_offset; $i <= count($new_indices_arr1); $i++) {

			$if_index = $new_indices_arr1[$i];

			if(!empty($interface_desc_a[$if_index])) {
				// If description of this interface is not empty, we want to
				// display it.  Else ignore it.
			
				$new_indices_arr2[$index] = $if_index;
				$index++; 
			}

		
			// We have reached the number of rows limit per page.
			if($index > $telix_ifs_view_num_rows) {
				break;
			}
		} 

		// If there is at least a row of data to be displayed, then we are
		// done.  Otherwise, try and display rows of data on page 1.
		if(!empty($new_indices_arr2)) {
			break;
		} else {

			if($_REQUEST["page_ifs_view"] == 1) {
				// This is the case where there is nothing 
				// to display on first page.
				return array(INTERFACE_INDICES_KEY=>array());
			}

			$_REQUEST["page_ifs_view"] = 1;
			$_SESSION["sess_router_ifs_view_current_page"] = $_REQUEST["page_ifs_view"];
		}
	}	


	// Extract required elements from other arrays using
	// interface index.	
	$new_desc_arr = array();
	$new_type_arr = array();
	$new_admin_status_arr = array();
	$new_op_status_arr = array();
	foreach($new_indices_arr2 as $key => $if_index) {
	
		// Process description array.
		$value = $interface_desc_a[$if_index];
		$new_desc_arr[$if_index] = $value;

		// Process type array.
		$value = $interface_type_a[$if_index];
		$new_type_arr[$if_index] = $value;

		// Process admin status array.
		$value = $interface_admin_status_a[$if_index];
		$new_admin_status_arr[$if_index] = $value;

		// Process operational status array.
		$value = $interface_op_status_a[$if_index];
		$new_op_status_arr[$if_index] = $value;

	}
	
	
	// Create a new interface status array based on processing performed
	// and return it.
	$new_interface_status_a = array();
	$new_interface_status_a[INTERFACE_INDICES_KEY] = $new_indices_arr2;
	$new_interface_status_a[INTERFACE_DESC_KEY] = $new_desc_arr;
	$new_interface_status_a[INTERFACE_TYPE_KEY] = $new_type_arr;
	$new_interface_status_a[INTERFACE_ADMIN_STATUS_KEY] = $new_admin_status_arr;
	$new_interface_status_a[INTERFACE_OP_STATUS_KEY] = $new_op_status_arr;
	return $new_interface_status_a;


}


/*
 * Purpose: 
 *    Get refresh interval for this page.
 *
 */
function get_refresh_interval() {

	global $settings, $config;


	// Check if the user has changed the refresh rate for the session.
	// The new value is set through the View Router Interfaces Status page and 
	// not the Settings/Telix page.
	if( isset($_REQUEST["new_refresh_rate"]) ) {
		
		// Check to make sure that the user entered a numeric value.
		// If a non-numeric value is entered, it is ignored.
		if((is_numeric($_REQUEST["new_refresh_rate"])) &&
               ($_REQUEST["new_refresh_rate"] >= 1) ) {
			$_SESSION["sess_ifs_view_page_refresh_rate"] = intval($_REQUEST["new_refresh_rate"]);
		} 
	}	


	// If refresh rate already set and set as session variable, use it.
	if( (isset($_SESSION["sess_ifs_view_page_refresh_rate"])) && 
          ($_SESSION["sess_ifs_view_page_refresh_rate"] >= 1) ) {
		$refresh["seconds"] = intval($_SESSION["sess_ifs_view_page_refresh_rate"]);
		$refresh["page"] = "telix_ifs_view.php";
		return $refresh;
	}


	include_once($config["library_path"] . "/database.php");

	// The condition for name column must match the field name specified in "telix_config_settings" 
	// function of plugings/telix/setup.php page.  That is, the field name assigned to Interface
	// Management page refresh rate.
	$sqlQuery = "select value from settings where name='telix_ifs_view_default_refresh_rate'";
	$default_refresh_rate = db_fetch_row($sqlQuery);

	// If user did not set default refresh rate or if refresh rate is set at
  	// less than or equals to 0, then factory default used.  Also remember
	// this value as session variable.
	if( (!isset($default_refresh_rate)) ||
          (!isset($default_refresh_rate["value"])) ||
	    (!is_numeric($default_refresh_rate["value"])) ||
          ($default_refresh_rate["value"] < 1) ) {
		
		// Remember the refresh rate.
		$_SESSION["sess_ifs_view_page_refresh_rate"] = FACTORY_DEFAULT_IFS_VIEW_REFRESH_RATE; 
		
		$refresh["seconds"] = $_SESSION["sess_ifs_view_page_refresh_rate"];
		$refresh["page"] = "telix_ifs_view.php";
		return $refresh;
	}
	
	
	// Display user-defined default.
	$_SESSION["sess_ifs_view_page_refresh_rate"] = intval($default_refresh_rate["value"]);
	$refresh["seconds"] = $_SESSION["sess_ifs_view_page_refresh_rate"];
	$refresh["page"] = "telix_ifs_view.php";
	return $refresh;

}


?>














